with Ada.Unchecked_Deallocation;
with System;
with Types; use Types;
with Tokens;

package Iirs is
   -- This package defines the semantic tree and functions to handle it.
   -- The tree is roughly based on IIR (Internal Intermediate Representation),
   -- [AIRE/CE Advanced Intermediate Representation with Extensibility,
   --  Common Environment.  http://www.vhdl.org/aire/index.html ]
   -- but oriented object features are not used, and sometimes, functions
   -- or fields have changed.

   -- Note: this tree is also used during syntaxic analysis, but with
   -- a little bit different meanings for the fields.
   -- The parser (parse package) build the tree.
   -- The semantic pass (sem, sem_expr, sem_name) transforms it into a
   -- semantic tree.

   -- Documentation:
   -- Only the semantic aspect is to be fully documented.
   -- The syntaxic aspect is only used between parse and sem.

   -- Each node of the tree is a record of type iir.  The record has only
   -- one discriminent, which contains the kind of the node.  There is
   -- currenlty no variant (but this can change, this is not public).

   -- The root of a semantic tree is a library_declaration.
   -- All the library_declarations are kept in a private list, held by
   -- package libraries.
   -- Exemple of a tree:
   --   library_declaration
   --   +-- design_file
   --       +-- design_unit
   --       |   +-- entity_declaration
   --       +-- design_unit
   --           +-- architecture_declaration
   -- ...

   -- Since the tree can represent all the libraries and their contents, it
   -- is not always loaded into memory.
   -- When a library is loaded, only library_declaration, design_file,
   -- design_unit and library_unit nodes are created.  When a design_unit is
   -- really loaded, the design_unit node is not replaced but modified (ie,
   -- access to this node are still valid).

   -- To add a new kind of node:
   --   the name should be of the form iir_kind_NAME
   --   add iir_kind_NAME in the definition of type iir_kind_type
   --   add a declaration of access type of name iir_kind_NAME_acc
   --   document the node below: grammar, methods.
   --   for each methods, add the name if the case statement in the body
   --     (this enables the methods)
   --   add an entry in create_iir and free_iir
   --   add an entry in disp_tree (debugging)

   -------------------------------------------------
   -- General methods (can be used on all nodes): --
   -------------------------------------------------

   -- Create a node of kind KIND.
   --   function Create_Iir (Kind: Iir_Kind) return Iir;
   --
   -- Deallocate a node.  Deallocate fields that where allocated by create_iir.
   --   procedure Free_Iir (Target: in out Iir);
   --
   -- Get the kind of the iir.
   -- See below for the (public) list of kinds.
   --   function Get_Kind (An_Iir: Iir) return Iir_Kind;

   -- Get the location of the node: ie the current position in the source
   -- file when the node was created.  This is a little bit fuzzy.
   --
   --   procedure Set_Location (Target: in out Iir; Location: Location_Type);
   --   function Get_Location (Target: in out Iir) return Location_Type;
   --
   --   function Get_Line_Number (Target: Iir) return Natural;
   --   function Get_Column_Number (Target: Iir) return natural;
   --   function Get_File_Name (Target: in Iir) return name_id;
   --
   -- Copy a location from a node to another one.
   --   procedure Location_Copy (Target: in out Iir; Src: in Iir);

   -- Get or Set info for a back-end.
   --   function  Get_Back_End_Info (Target: in Iir) return System.Address;
   --   procedure Set_Back_End_Info (Target: in out Iir; Addr: System.Address);

   -- The next line marks the start of the node description.
   -- Start of Iir_Kind.

   -------------------------------------------------
   -- A set of methods are associed with a kind.  --
   -------------------------------------------------

   -- Iir_Kind_Design_File
   -- LRM93 11
   -- DESIGN_FILE ::= DESIGN_UNIT { DESIGN_UNIT}
   --
   -- The library containing this design file.
   --   Get/Set_Parent (Parent)
   --
   -- Get the list of unit contained in the file.
   --   Get/Set_Design_Unit_Chain (Field1)
   --
   --   Get/Set_File_Time_Stamp (Field2)
   --
   --   Get/Set_Analysis_Time_Stamp (Field3)
   --
   -- Flag used during elaboration.  Set when the file was already seen.
   --   Get/Set_Elab_Flag (Flag3)
   --
   -- Identifier for the design file file name and dirname.
   --   Get/Set_Design_File_Filename (Nbr3)
   --   Get/Set_Design_File_Directory (Nbr2)

   -- Iir_Kind_Design_Unit
   -- LRM93 11
   -- DESIGN_UNIT ::= CONTEXT_CLAUSE LIBRARY_UNIT
   --
   -- The design_file containing this design unit.
   --   Get/Set_Design_File (Parent)
   --   Get/Set_Parent (Parent)
   --
   --   Get/Set_Identifier (Ident)
   --
   -- Get the chain of context clause.
   --   Get_Context_Items (Field1)
   --
   -- Get the list of design units that must be analysed before this unit.
   -- See LRM93 11.4 for the rules defining the order of analysis.
   --   Get_Dependence_List (Field3)
   --
   -- Get/Set the library unit, which can be an entity, an architecture,
   -- a package, a package body or a configuration.
   --   Get/Set_Library_Unit (Field2)
   --
   -- Get/Set the loaded flag, which indicates whether this design unit is in
   -- memory or not.
   --   Get/Set_Loaded_Flag (Flag1)
   --
   -- Flag used during elaboration.  Set when the file was already seen.
   --   Get/Set_Elab_Flag (Flag3)
   --
   --   Get/Set_Visible_Flag (Flag4)
   --
   -- Set the line and the offset in the line, only for the library manager.
   -- This is valid until the file is really loaded in memory.  On loading,
   -- location will contain all this informations.
   -- Get/Set_Pos_Line_Off (Field1,nbr2,nbr3)(Field1,field6,field7)
   --
   -- Get/Set the date.
   -- This is a symbolic date, only used as a order of analysis of design
   -- units.
   --   Get/Set_Date (Field5)
   --
   --   Get/Set_Attribute_Value_List (Attr)

   -- Iir_Kind_Library_Clause
   -- Note: a library_clause node is created for every logical_name.
   -- As a consequence, the scope of the library starts after the logical_name
   -- and not after the library_clause.  However, since an identifier
   -- can only be used as a logical_name, and since the second occurence has
   -- no effect, this is correct.
   --
   --   Get/Set_Identifier (Ident)
   --
   --   Get/Set_Library_Declaration (Field2)
   --
   --   Get/Set_Visible_Flag (Flag4)

   --------------
   -- Literals --
   --------------

   -- Iir_Kind_Character_Literal
   -- Iir_Kind_String_Literal
   -- Iir_Kind_Bit_String_Literal
   --
   -- Type of the literal.  Note: for a (bit_)string_literal, the type must be
   -- computed during semantization.  Roughly speaking, this is possible since
   -- integer type range constraint are locally static.
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Identifier (Ident)
   --
   -- Used for computed literals.  Literal_Origin contains the expression whose
   -- value was computed during analysis and replaces the expression.
   --   Get/Set_Literal_Origin (Field3)
   --
   -- The logarithm of the base (1, 3 or 4) of a bit string.
   --   Get/Set_Bit_String_Base (Nbr2)
   --
   -- Only when the string is interpreted as a operator_symbol.
   --   Get/Set_Named_Entity (Field5)
   --
   -- For bit string only:
   -- Enumeration literal which correspond to '0' and '1'.
   -- This cannot be defined only in the enumeration type definition, due to
   -- possible aliases.
   --   Get/Set_Bit_String_0 (Field4)
   --   Get/Set_Bit_String_1 (Field5)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)

   -- Iir_Kind_Integer_Literal
   --   Get/Set_Type (Field1)
   --
   -- Get/Set the value of the integer.
   --   Get/Set_Value (Nbr3)
   --
   --   Get/Set_Literal_Origin (Field3)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)

   -- Iir_Kind_Floating_Point_Literal
   --   Get/Set_Type (Field1)
   --
   -- Get/Set the value of the literal.
   --   Get/Set_Fp_Value (Nbr3)
   --
   --   Get/Set_Literal_Origin (Field3)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)

   -- Iir_Kind_Null_Literal
   -- The null literal, which can be a disconnection or a null access.
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)

   -- Iir_Kind_Physical_Int_Literal
   -- Iir_Kind_Physical_Fp_Literal
   --
   --   Get/Set_Type (Field1)
   --
   -- Get/Set the physical unit of the literal.
   --   Get/Set_Name (Field2)
   --
   -- The multiplicand.
   --   Get/Set_Value (Nbr3)
   --   Get/Set_Fp_Value (Nbr3)
   --
   --   Get/Set_Literal_Origin (Field3)
   --
   -- Must be set to locally except for time literal, which is globally.
   --   Get/Set_Expr_Staticness (Staticness_1)

   -- Iir_Kind_Simple_Aggregate
   -- This node can only be generated by evaluation: it is an unidimentional
   -- positional aggregate.
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Literal_Origin (Field3)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)
   --
   -- List of elements
   --   Get/Set_Simple_Aggregate_List (Field2)

   ------------
   -- Tuples --
   ------------

   -- Iir_Kinds_Association_Element
   -- Iir_Kind_Association_Element_By_Expression
   -- Iir_Kind_Association_Element_Open
   -- Iir_Kind_Association_Element_By_Individual
   -- These are used for association element of an association list with
   -- an interface (ie subprogram call, port map, generic map).
   --
   --   Get/Set_Formal (Field1)
   --
   -- Only for iir_kind_association_element_by_expression
   --   Get/Set_Actual (Field2)
   --   Get/Set_In_Conversion (Field4)
   --   Get/Set_Out_Conversion (Field5)
   --
   -- Only for iir_kind_association_element_by_individual
   --   Get/Set_Actual_Type (Field2)
   --   Get/Set_Individual_Association_Chain (Field4)
   --
   -- Get/Set the associated formal (from an interface list).
   --   Get/Set_Associated_Formal (Field3)
   --
   -- Get/Set the whole association flag (true if the formal is associated in
   -- whole, see LRM 4.3.2.2)
   --   Get/Set_Whole_Association_Flag (Flag1)

   -- Iir_Kind_Designator_By_All
   -- Iir_Kind_Designator_By_Others

   -- Iir_Kind_Proxy
   -- A proxy is used to avoid duplication of a node.
   -- Ex: instead of copying a default value of an insterface in the subprogram
   -- call, a proxy is used.  The default value can't be so easily aliased
   -- due to annotation.
   --
   -- Create a proxy for PROXY.
   --   function Create_Proxy (Proxy: Iir) return Iir_Proxy;
   --
   -- Get/Set the value of the proxy.
   --   Get/Set_Proxy (Field1)

   -- Iir_Kind_Waveform_Element
   --
   --   Get/Set_We_Value (Field1)
   --
   --   Get/Set_Time (Field2)

   -- Iir_Kind_Conditional_Waveform
   --
   --   Get/Set_Condition (Field1)
   --
   --   Get/Set_Waveform_Chain (Field3)

   -- Iir_Kinds_Choice
   -- These are elements of an choice chain, which is used for
   -- case_statement, concurrent_select_signal_assignment, aggregates.
   --
   -- Get/Set was is associated with the choice.  This can be:
   --  * a waveform_chain for a concurrent_select_signal_assignment,
   --  * an expression for an aggregate,
   --  * a sequential statement list for a case_statement.
   -- For a list of choices, only the first one is associated.
   --   Get/Set_Associated (Field1)
   --
   -- For by_list, by_expression, by_range: staticness of the choice.
   --   Get/Set_Choice_Staticness (Staticness_2)

   -- Iir_Kind_Choice_By_Others
   -- See iir_kinds_by_choices.

   -- Iir_Kind_Choice_By_None
   -- See iir_kinds_by_choices.
   -- positionnal association (used only in aggregates).

   -- Iir_Kind_Choice_By_Expression
   -- See iir_kinds_by_choices.
   --
   --   Get/Set_Expression (Field3)

   -- Iir_Kind_Choice_By_Range
   -- See iir_kinds_by_choices.
   --
   -- Get/Set the range.
   --   Get/Set_Expression (Field3)

   -- Iir_Kind_Choice_By_Name.
   -- See iir_kinds_by_choices.
   --
   -- Get/Set the name.
   --   Get/Set_Name (Field2)

   -- Iir_Kind_Entity_Aspect_Entity
   --
   -- Parse: a name
   -- Sem: a design unit
   --   Get/Set_Entity (Field1)
   --
   -- parse: a simple name.
   -- sem: an architecture declaration or NULL_IIR.
   --   Get/Set_Architecture (Field2)

   -- Iir_Kind_Entity_Aspect_Open

   -- Iir_Kind_Entity_Aspect_Configuration
   --
   -- Parse: a name
   -- Sem: a design unit
   --   Get/Set_Configuration (Field1)

   -- Iir_Kind_Block_Configuration
   -- Note: for default block configurations of iterative generate statement,
   -- the block specification is a selected_name, whose identifier is others.
   --   Get/Set_Block_Specification (Field1)
   --
   -- Single linked list of block configuration that apply to the same
   -- for scheme generate block.
   --   Get/Set_Prev_Block_Configuration (Field4)
   --
   --   Get_Declaration_Chain (Field3)
   --
   --   Get/Set_Configuration_Item_Chain (Field2)

   -- Iir_Kind_Binding_Indication
   -- The entity aspect.
   -- It is a iir_kind_entity_aspect_entity, iir_kind_entity_aspect_open or
   -- iir_kind_entity_aspect_configuration.  This may be transformed into a
   -- declaration by semantic.
   --   Get/Set_Entity_Aspect (Field4)
   --
   --   Get/Set_Generic_Map_Aspect_List (Field5)
   --
   --   Get/Set_Port_Map_Aspect_List (Field3)
   --
   --   Get/Set_Default_Entity_Aspect (Field1)
   --
   --   Get/Set_Default_Generic_Map_Aspect_Chain (Field2)
   --
   --   Get/Set_Default_Port_Map_Aspect_Chain (Field6)

   -- Iir_Kind_Component_Configuration
   -- Iir_Kind_Configuration_Specification
   --
   -- The declaration containing this type declaration.
   --   Get/Set_Parent (Parent)
   --
   --   Get/Set_Component_Name (Field4)
   --
   -- Must be one of designator_list, designator_by_others or
   -- designator_by_all.
   --   Get/Set_Instantiation_List (Field1)
   --
   -- Only for a component_configuration
   --   Get/Set_Block_Configuration (Field2)
   --
   --   Get/Set_Binding_Indication (Field3)

   -- Iir_Kind_Disconnection_Specification
   --
   -- The declaration containing this type declaration.
   --   Get/Set_Parent (Parent)
   --
   --   Get/Set_Signal_List (Field2)
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Expression (Field3)

   -- Iir_Kind_Block_Header
   --
   --   Get/Set_Port_Chain (Field1)
   --
   --   Get/Set_Generic_Chain (Field4)
   --
   --   Get/Set_Generic_Map_Aspect_List (Field5)
   --
   --   Get/Set_Port_Map_Aspect_List (Field3)

   -- Iir_Kind_Entity_Class
   --   Get/Set_Entity_Kind (Nbr3)

   -- Iir_Kind_Attribute_Specification
   --
   --   Get/Set_Attribute_Designator (Field2)
   --
   --   Get/Set_Expression (Field3)
   --
   --   Get/Set_Entity_Name_List (Field1)
   --
   --   Get/Set_Entity_Kind (Nbr3)
   --
   --   Get/Set_Attribute_Value_List (Attr)

   -- Iir_Kind_Attribute_Value
   -- An attribute value is the element of the list of attribute of an entity,
   -- marking the entity as decorated by the attribute.
   -- This node is built only by sem.
   -- In fact, the node is member of the list of attribute of an entity, and
   -- of the list of entity of the attribute specification.
   -- This makes elaboration (and more precisely, expression evaluation)
   -- easier.
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Attribute_Specification (Field2)
   --
   --   Get/Set_Designated_Entity (Field3)
   --
   --   Get/Set_Base_Name (Field4)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)
   --
   --   Get/Set_Name_Staticness (Staticness_2)

   -- Iir_Kind_Selected_Element
   -- A record element selection.
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Prefix (Field3)
   --
   --   Get/Set_Selected_Element (Field2)
   --
   --   Get/Set_Base_Name (Field4)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)
   --
   --   Get/Set_Name_Staticness (Staticness_2)

   -- Iir_Kind_Implicit_Dereference
   -- Iir_Kind_Dereference
   -- An implicit access dereference.
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Prefix (Field3)
   --
   --   Get/Set_Base_Name (Field4)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)
   --
   --   Get/Set_Name_Staticness (Staticness_2)

   --  Iir_Kind_Signature
   --
   --   Get/Set_Type_Marks_List (Field3)
   --
   --  Used only for attribute specification.
   --   Get/Set_Name (Field2)
   --
   --   Get/Set_Return_Type (Field1)

   -----------
   -- Lists --
   -----------

   -- Iir_Kinds_List
   -- Lists of elements.
   -- index is 0 .. nbr_elements - 1.
   --
   -- Append an element to (the end of) the list.
   --   procedure Append_Element (List: in Iir; Element: Iir);
   --
   -- Get the N th element in list, starting from 0.
   -- Return an access to the element or null_iir, if beyond bounds.
   --   function Get_Nth_Element (List: in Iir; N: Natural) return Iir;
   --
   -- Return the last element of the list, or null_iir.
   --   function Get_Last_Element (List: in Iir) return Iir;
   --
   -- Return the first element of the list, or null_iir.
   --   function Get_First_Element (List: in Iir) return Iir;
   --
   -- Replace an element selected by position.
   --   procedure Replace_Nth_Element (List: in Iir_List; N: Natural; El:Iir);
   --
   -- Add (append) an element only if it was not already present in the list.
   -- Return its position.
   --   procedure Add_Element (List: in Iir; El: Iir; Position: out integer);
   --   procedure Add_Element (List: in Iir_List; El: Iir);
   --
   -- Return the number of elements in the list.
   -- This is also 1 + the position of the last element.
   --   function Get_Nbr_Elements (List: in Iir_List) return Natural;
   --
   -- Set the number of elements in the list.
   -- Can be used only to shrink the list.
   --   procedure Set_Nbr_Elements (List: in Iir_List; N: Natural);
   --
   -- Remove an element from the list.
   --   procedure remove_Nth_Element (List: in Iir_List; N: Natural);
   --
   -- Return the position of the last element.
   -- Return -1 if the list is empty.
   --   function Get_Last_Element_Position (List: in Iir_List) return Integer;
   --
   -- Empty the list.
   -- This is also set_nbr_elements (list, 0);
   --   procedure Empty_List (List: in Iir_List);
   --
   -- Alias a list.  TARGET must be empty.
   --   procedure Alias_List (Target: in out Iir; Source: in Iir);

   ------------------
   -- Declarations --
   ------------------

   -- Iir_Kind_Entity_Declaration
   --   Get/Set_Identifier (Ident)
   --
   --   Get/Set_Design_Unit (Parent)
   --
   --   Get_Declaration_Chain (Field3)
   --
   --   Get/Set_Port_Chain (Field1)
   --
   --   Get/Set_Generic_Chain (Field4)
   --
   --   Get/Set_Concurrent_Statement_Chain (Field2)
   --
   --   Get/Set_Is_Within_Flag (Flag5)

   -- Iir_Kind_Architecture_Declaration
   --   Get/Set_Identifier (Ident)
   --
   --   Get/Set_Design_Unit (Parent)
   --
   --   Get_Declaration_Chain (Field3)
   --
   --   Get/Set_Is_Within_Flag (Flag5)
   --
   --   Get/Set_Foreign_Flag (Flag3)
   --
   -- Set the entity of an architecture.
   -- Before the semantic pass, it can be a name.
   --   Get/Set_Entity (Field1)
   --
   --   Get/Set_Concurrent_Statement_Chain (Field2)
   --
   -- The default configuration created by canon.  This is a design unit.
   --   Get/Set_Default_Configuration_Declaration (Field4)

   -- Iir_Kind_Configuration_Declaration
   --   Get/Set_Identifier (Ident)
   --
   --   Get/Set_Design_Unit (Parent)
   --
   --   Get_Declaration_Chain (Field3)
   --
   -- Set the entity of a configuration (a design_unit)
   -- Before the semantic pass, it can be an identifier.
   --   Get/Set_Entity (Field1)
   --
   --   Get/Set_Block_Configuration (Field2)

   -- Iir_Kind_Package_Declaration
   --   Get/Set_Identifier (Ident)
   --
   --   Get/Set_Design_Unit (Parent)
   --
   --   Get_Declaration_Chain (Field3)
   --
   --   Get/Set_Need_Body (Flag1)

   -- Iir_Kind_Package_Body
   -- Note: a body is not a declaration, that's the reason why there is no
   -- _declaration suffix in the name.
   --
   --   Get/Set_Package (Field1)
   --
   --   Get/Set_Design_Unit (Parent)
   --
   --   Get/Set_Identifier (Ident)
   --
   --   Get_Declaration_Chain (Field3)

   -- Iir_Kind_Library_Declaration
   -- This node is used to contain all a library.  Only internaly used.
   -- Name (identifier) of the library.
   --   Get/Set_Identifier (Ident)
   --
   --   Get/Set_Library_Directory (Nbr2)
   --
   -- Design files in the library.
   --   Get/Set_Design_File_Chain (Field1)
   --
   --   Get/Set_Date (Field5)
   --
   --   Get/Set_Visible_Flag (Flag4)

   -- Iir_Kind_Component_Declaration
   --   Get/Set_Identifier (Ident)
   --
   -- The declaration containing this type declaration.
   --   Get/Set_Parent (Parent)
   --
   --   Get/Set_Port_Chain (Field1)
   --
   --   Get/Set_Generic_Chain (Field4)
   --
   --   Get/Set_Attribute_Value_List (Attr)

   -- Iir_Kind_Object_Alias_Declaration
   --   Get/Set_Identifier (Ident)
   --
   -- The declaration containing this type declaration.
   --   Get/Set_Parent (Parent)
   --
   --   Get/Set_Name (Field2)
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Base_Name (Field4)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)
   --
   --   Get/Set_Name_Staticness (Staticness_2)

   -- Iir_Kind_Non_Object_Alias_Declaration
   --   Get/Set_Identifier (Ident)
   --
   -- The declaration containing this type declaration.
   --   Get/Set_Parent (Parent)
   --
   --   Get/Set_Name (Field2)
   --
   --   Get/Set_Signature (Field4)

   -- Iir_Kind_Anonymous_Type_Declaration
   --
   -- Used for informative purpose only.
   --   Get/Set_Identifier (Ident)
   --
   --   Get/Set_Parent (Parent)
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Subtype_Definition (Field2)

   -- Iir_Kind_Type_Declaration
   --
   -- Name of the type.
   --   Get/Set_Identifier (Ident)
   --
   -- The declaration containing this type declaration.
   --   Get/Set_Parent (Parent)
   --
   -- Definition of the type.
   -- Note: the type definition can be a real type (unconstrained array,
   -- enumeration, file, record, access) or a subtype (integer, floating
   -- point).
   -- The parser set this field to null_iir for an incomplete type declaration.
   -- This field is set to an incomplete_type_definition node when first
   -- semantized.
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Subtype_Definition (Field2)
   --
   --   Get/Set_Attribute_Value_List (Attr)

   -- Iir_Kind_Subtype_Declaration
   --
   --   Get/Set_Identifier (Ident)
   --
   -- The declaration containing this subtype declaration.
   --   Get/Set_Parent (Parent)
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Attribute_Value_List (Attr)

   -- Iir_Kind_Signal_Interface_Declaration
   -- Iir_Kind_Constant_Interface_Declaration
   -- Iir_Kind_Variable_Interface_Declaration
   -- Iir_Kind_File_Interface_Declaration
   --
   -- Note: If type is an iir_kind_proxy node, then type *and* default value
   -- (if any) must be extracted from proxy.
   --
   --   Get/Set_Identifier (Ident)
   --
   -- Get/Set the parent of an interface declaration.
   -- The parent is an entity declaration, a subprogram specification, a
   -- component declaration, a loop statement, a block declaration or ??
   -- Useful to distinguish a port and an interface.
   --   Get/Set_Parent (Parent)
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Mode (Odigit_2)
   --
   --   Get/Set_Lexical_Layout (Odigit_1)
   --
   -- For iir_kind_signal_interface_declaration:
   --   Get/Set_Signal_Kind (State1)
   --
   --   Get/Set_Has_Disconnect_Flag (Flag1)
   --
   -- Must always be null_iir for iir_kind_file_interface_declaration.
   --   Get/Set_Default_Value (Field2)
   --
   --   Get/Set_Attribute_Value_List (Attr)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)

   -- Iir_Kind_Function_Declaration
   -- Iir_Kind_Procedure_Declaration
   --
   -- Subprogram declaration.
   --
   --   Get/Set_Identifier (Ident)
   --
   -- The declaration containing this type declaration.
   --   Get/Set_Parent (Parent)
   --
   --   Get_Interface_Declaration_Chain (Field4)
   --
   --   Get/Set_Foreign_Flag (Flag3)
   --
   -- Corresponding body.
   --   Get/Set_Subprogram_Body (Field2)
   --
   -- Only for iir_kind_function_declaration.
   --   Get/Set_Return_Type (Field1)
   --
   --   Get/Set_Driver_List (Field5)
   --
   --   Get/Set_Attribute_Value_List (Attr)
   --
   --   Get/Set_Is_Within_Flag (Flag5)

   -- Iir_Kind_Function_Body
   -- Iir_Kind_Procedure_Body
   --
   -- Parse always put a declaration before a body.
   -- Sem will remove this declaration if there is a forward declaration.
   --
   -- Corresponding specification for a body
   --   Get/Set_Subprogram_Specification (Field4)
   --
   --   Get_Declaration_Chain (Field3)
   --
   --   Get/Set_Sequential_Statement_Chain (Field2)

   -- Iir_Kind_Implicit_Procedure_Declaration
   -- Iir_Kind_Implicit_Function_Declaration
   --
   -- This node contains a subprogram_declaration that was implicitly defined
   -- just after a type declaration.
   -- This declaration is inserted by sem.
   --
   -- Name of the subprogram.
   -- FIXME: change name for the later use.
   --   Get/Set_Identifier (Ident)
   --
   --   Get_Interface_Declaration_Chain (Field4)
   --
   -- Only for iir_kind_implicit_function_declaration
   --   Get/Set_Return_Type (Field1)
   --
   --   Get/Set_Implicit_Definition (Field7)
   --
   --   Get/Set_Type_Reference (Field2)
   --
   --   Get/Set_Attribute_Value_List (Attr)
   --
   --   Get/Set_Is_Within_Flag (Flag5)

   -- Iir_Kind_Signal_Declaration
   -- Iir_Kind_Guard_Signal_Declaration
   --
   -- Get/Set_Identifier (Ident)
   --
   -- The declaration containing this type declaration.
   --   Get/Set_Parent (Parent)
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Signal_Kind (State1)
   --
   -- Default value of the signal (not available for an implicit guard signal)
   --   Get/Set_Default_Value (Field2)
   --
   -- Expression of guard signal (available only for implicit guard signal)
   --   Get/Set_Guard_Expression (Field2)
   --
   --   Get/Set_Base_Name (Field4)
   --
   -- Only for an implicit guard signal:
   --   Get/Set_Block_Statement (Field3)
   --
   -- For a non-resolved signal: null_iir if the signal has no driver, or
   -- a process/concurrent_statement for which the signal should have a
   -- driver.  This is used to catch at analyse time unresolved signals with
   -- several drivers.
   --   Get/Set_Signal_Driver (Field5)
   --
   -- Get/Set the sensitivity list for the implicit guard signal.
   --   Get/Set_Guard_Sensitivity_List (Field5)
   --
   --   Get/Set_Has_Disconnect_Flag (Flag1)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)
   --
   --   Get/Set_Attribute_Value_List (Attr)

   -- Iir_Kind_Constant_Declaration
   -- Iir_Kind_Iterator_Declaration
   --
   --   Get/Set_Identifier (Ident)
   --
   -- The declaration containing this type declaration.
   --   Get/Set_Parent (Parent)
   --
   --   Get/Set_Type (Field1)
   --
   -- Only for iir_kind_constant_declaration.
   -- Default value of a deferred constant points to the full constant
   -- declaration.
   --   Get/Set_Default_Value (Field2)
   --
   --   Get/Set_Base_Name (Field4)
   --
   -- Only for iir_kind_constant_declaration.
   -- Summary:
   -- | constant C1 : integer;          -- Deferred declaration (in a package)
   -- |  constant C2 : integer := 4;     -- Declaration
   -- |  constant C1 : integer := 3;     -- Full declaration (in a body)
   -- | NAME   Deferred_declaration  Deferred_declaration_flag
   -- |  C1      Null_iir or C1' (*)     True
   -- |  C2      Null_Iir                False
   -- |  C1'     C1                      False
   -- |(*): Deferred_declaration is Null_Iir as long as the full declaration
   -- |   has not been analyzed.
   --
   --   Get/Set_Deferred_Declaration (Field3)
   --
   --   Get/Set_Deferred_Declaration_Flag (Flag1)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)
   --
   --   Get/Set_Attribute_Value_List (Attr)

   -- Iir_Kind_Variable_Declaration
   --
   --   Get/Set_Identifier (Ident)
   --
   -- The declaration containing this type declaration.
   --   Get/Set_Parent (Parent)
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Default_Value (Field2)
   --
   --   Get/Set_Base_Name (Field4)
   --
   -- True if the variable is a shared variable.
   --   Get/Set_Shared_Flag (Flag2)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)
   --
   --   Get/Set_Attribute_Value_List (Attr)

   -- Iir_Kind_File_Declaration
   --
   --   Get/Set_Identifier (Ident)
   --
   -- The declaration containing this type declaration.
   --   Get/Set_Parent (Parent)
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_File_Logical_Name (Field2)
   --
   --   Get/Set_Base_Name (Field4)
   --
   -- This is used only in vhdl 87.
   --   Get/Set_Mode (Odigit_2)
   --
   -- This is not used in vhdl 87.
   --   Get/Set_File_Open_Kind (Field3)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)
   --
   --   Get/Set_Attribute_Value_List (Attr)

   -- Iir_Kind_Element_Declaration
   --
   --   Get/Set_Identifier (Ident)
   --
   --   Get/Set_Type (Field1)
   --
   -- Return the position of the element in the record, starting from 0 for the
   -- first record element, increasing by one for each successive element.
   --   Get/Set_Element_Position (Field6)

   -- Iir_Kind_Attribute_Declaration
   --
   --   Get/Set_Identifier (Ident)
   --
   --   Get/Set_Type (Field1)

   -- Iir_Kind_Group_Template_Declaration
   --   Get/Set_Identifier (Ident)
   --
   -- List of entity class entry.
   -- To handle `<>', the last element of the list can be an entity_class of
   -- kind tok_box.
   --   Get/Set_Entity_Class_Entry_Chain (Field1)

   -- Iir_Kind_Group_Declaration
   --   Get/Set_Identifier (Ident)
   --
   -- The declaration containing this type declaration.
   --   Get/Set_Parent (Parent)
   --
   -- Get/Set group template name.
   --   Get/Set_Name (Field2)
   --
   -- List of constituent.
   --   Get_Group_Constituent_List (Field1)
   --
   --   Get/Set_Attribute_Value_List (Attr)

   -- Iir_Kind_Use_Clause
   --   Get/Set_Selected_Name (Field1)
   --
   --   Get/Set_Use_Clause_Chain (Field2)


   ----------------------
   -- type definitions --
   ----------------------

   -- Iir_Kinds_Type_And_Subtype_Definition
   --
   -- Type_Declarator:
   -- Points to the type declaration or subtype declaration that has created
   -- this definition. For some types, such as integer and floating point
   -- types, both type and subtype points to the declaration.
   -- However, there are cases where a type definition doesn't point to
   -- a declarator: anonymous subtype created by index contraints, or
   -- anonymous subtype created by an object declaration.
   -- Note: a type definition cannot be anoynymous.
   --   Get/Set_Type_Declarator (Field1)
   --
   -- Get/Set the base type.
   -- For a subtype, it returns the type.
   -- For a type, it must return the type itself.
   --   Get/Set_Base_Type (Field4)
   --
   -- Get/Set the staticness of a type, according to LRM93 7.4.1.
   -- Note: These types definition are always locally static:
   -- enumeration, integer, floating.
   -- However, their subtype are not necessary locally static.
   --   Get/Set_Type_Staticness (Staticness_1)
   --
   -- Get/Set the resolved flag of a subtype, according to LRM93 2.4
   --   Get/Set_Resolved_Flag (Flag1)
   --
   -- Get/Set the signal_type flag of a type definition.
   -- It is true when the type can be used for a signal.
   --   Get/Set_Signal_Type_Flag (Flag2)

   -- Iir_Kind_Enumeration_Type_Definition
   -- Fields: type_staticness.
   --   Get/Set_Type_Declarator (Field1)
   --
   --   Get/Set_Base_Type (Field4)
   --
   -- Get the range of the type (This is just an ascending range from the
   -- first literal to the last declared literal).
   --   Get/Set_Range_Constraint (Field2)
   --
   -- Return the list of literals.  This list is created when the node is
   -- created.
   --   Get_Enumeration_Literal_List (Field3)
   --
   --   Get/Set_Resolved_Flag (Flag1)
   --
   --   Get/Set_Signal_Type_Flag (Flag2)

   -- Iir_Kind_Enumeration_Literal
   --
   -- Nota: two literals of the same type are equal iff their value is the
   -- same; in other words, there may be severals literals with the same value.
   --
   --   Get/Set_Identifier (Ident)
   --
   -- Note: Get/Set_Return_Type is an alias of Get/Set_Type.
   --   Get/Set_Type (Field1)
   --
   -- The value of an enumeration literal is the position.
   --   Get/Set_Value (Nbr3)
   --
   --   Get/Set_Literal_Origin (Field3)
   --
   -- The declaration of the literal.  If LITERAL_ORIGIN is not set, then this
   -- is the node itself, else this is the literal defined.
   --   Get/Set_Enumeration_Decl (Field2)
   --
   --   Get/Set_Base_Name (Field4)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)
   --
   --   Get/Set_Name_Staticness (Staticness_2)
   --
   --   Get/Set_Attribute_Value_List (Attr)

   -- Iir_Kind_Physical_Type_Definition
   --
   --   Get/Set_Type_Declarator (Field1)
   --
   --   Get/Set_Base_Type (Field4)
   --
   --   Get/Set_Primary_Unit (Field2)
   --
   --   Get_Unit_Chain (Field3)
   --
   --   Get/Set_Resolved_Flag (Flag1)
   --
   --   Get/Set_Signal_Type_Flag (Flag2)

   -- Iir_Kind_Unit_Declaration
   --
   --   Get/Set_Identifier (Ident)
   --
   --   Get/Set_Type (Field1)
   --
   -- Definition of the physical unit.  Is null_iir for the primary unit.
   --   Get/Set_Multiplier (Field2)
   --
   -- Value based on the primary unit.  1 for the primary unit.
   --   Get/Set_Value (Nbr3)
   --
   --   Get/Set_Attribute_Value_List (Attr)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)

   -- Iir_Kind_Integer_Type_Definition.
   -- Iir_Kind_Floating_Type_Definition.
   --
   -- Get/Set the declarator that has created this integer type.
   --   Get/Set_Type_Declarator (Field1)
   --
   --   Get/Set_Base_Type (Field4)
   --
   -- Type staticness is always locally.
   --   Get/Set_Type_Staticness (Staticness_1)
   --
   --   Get/Set_Resolved_Flag (Flag1)
   --
   --   Get/Set_Signal_Type_Flag (Flag2)

   -- Iir_Kind_Array_Type_Definition.
   -- This defines an unconstrained array type.
   --
   --   Get/Set_Type_Declarator (Field1)
   --
   --   Get/Set_Base_Type (Field4)
   --
   --   Get/Set_Element_Subtype (Field3)
   --
   --   Get_Index_List (Field2)
   --
   --   Get/Set_Type_Staticness (Staticness_1)
   --
   --   Get/Set_Resolved_Flag (Flag1)
   --
   --   Get/Set_Signal_Type_Flag (Flag2)

   -- Iir_Kind_Incomplete_Type_Definition
   -- Type definition for an incomplete type.  This is created during the
   -- semantisation of the incomplete type declaration.
   --
   -- Set to the incomplete type declaration when semantized, and set to the
   -- complete type declaration when the latter one is semantized.
   --   Get/Set_Type_Declarator (Field1)
   --
   --   Get_Incomplete_Type_List (Field2)
   --
   --   Get/Set_Base_Type (Field4)
   --
   --   Get/Set_Type_Staticness (Staticness_1)
   --
   --   Get/Set_Resolved_Flag (Flag1)
   --
   --   Get/Set_Signal_Type_Flag (Flag2)

   -------------------------
   -- subtype definitions --
   -------------------------

   -- Iir_Kind_Enumeration_Subtype_Definition
   -- Iir_Kind_Integer_Subtype_Definition
   -- Iir_Kind_Floating_Subtype_Definition
   -- Iir_Kind_Physical_Subtype_Definition
   --
   -- Fields: type_staticness.
   --
   --   Get/Set_Type_Declarator (Field1)
   --
   --   Get/Set_Base_Type (Field4)
   --
   --   Get/Set_Range_Constraint (Field2)
   --
   --   Get/Set_Type_Mark (Field6)
   --
   --   Get/Set_Resolution_Function (Field5)
   --
   --   Get/Set_Resolved_Flag (Flag1)
   --
   --   Get/Set_Signal_Type_Flag (Flag2)

   -- Iir_Kind_Access_Subtype_Definition
   --
   --   Get/Set_Type_Staticness (Staticness_1)
   --
   --   Get/Set_Type_Declarator (Field1)
   --
   --   Get/Set_Base_Type (Field4)
   --
   --   Get/Set_Type_Mark (Field6)
   --
   -- Note: no resolution function for access subtype.
   --
   --   Get/Set_Resolved_Flag (Flag1)
   --
   --   Get/Set_Signal_Type_Flag (Flag2)

   -- Iir_Kind_Record_Subtype_Definition
   --
   -- Fields: type_staticness.
   --
   --   Get/Set_Type_Declarator (Field1)
   --
   --   Get/Set_Base_Type (Field4)
   --
   --   Get/Set_Type_Mark (Field6)
   --
   --   Get/Set_Resolution_Function (Field5)
   --
   --   Get/Set_Resolved_Flag (Flag1)
   --
   --   Get/Set_Signal_Type_Flag (Flag2)

   -- Iir_Kind_Array_Subtype_Definition
   -- Iir_Kind_Unconstrained_Array_Subtype_Definition
   --
   -- Iir_Kind_Array_Subtype_definition defineds defines a constrained array
   -- subtype, which *must* be a subtype of an iir_array_type_definition.
   --
   -- Iir_Kind_Unconstrained_Array_Subtype_Definition defines a
   -- unconstrained array subtype, which *must* be a subtype of an
   -- iir_array_type_definition.  The only way to create such a
   -- subtype is via a subtype declaration, without adding
   -- constraints.
   --
   --   Get/Set_Type_Declarator (Field1)
   --
   --   Get/Set_Base_Type (Field4)
   --
   --   Get/Set_Type_Mark (Field6)
   --
   --   Get_Index_List (Field2)
   --
   --   Get/Set_Element_Subtype (Field3)
   --
   --   Get/Set_Resolution_Function (Field5)
   --
   --   Get/Set_Type_Staticness (Staticness_1)
   --
   --   Get/Set_Resolved_Flag (Flag1)
   --
   --   Get/Set_Signal_Type_Flag (Flag2)

   -- Iir_Kind_Record_Type_Definition
   --
   --   Get/Set_Type_Declarator (Field1)
   --
   --   Get/Set_Base_Type (Field4)
   --
   --   Get/Set_Type_Staticness (Staticness_1)
   --
   --   Get/Set_Element_Declaration_Chain (Field2)
   --
   --   Get/Set_Number_Element_Declaration (Nbr2)
   --
   --   Get/Set_Resolved_Flag (Flag1)
   --
   --   Get/Set_Signal_Type_Flag (Flag2)

   -- Iir_Kind_Range_Expression
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Direction (State1)
   --
   --   Get/Set_Left_Limit (Field2)
   --
   --   Get/Set_Right_Limit (Field3)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)

   -- Iir_Kind_Subtype_Definition
   -- Such a node is only created by parse and transformed into the correct
   -- kind (enumeration_subtype, integer_subtype...) by sem.
   --
   --   Get/Set_Range_Constraint (Field2)
   --
   --   Get/Set_Base_Type (Field4)

   -- Iir_Kind_Access_Type_Definition
   --
   --   Get/Set_Type_Declarator (Field1)
   --
   --   Get/Set_Base_Type (Field4)
   --
   --   Get/Set_Designated_Type (Field2)
   --
   -- Only for access_subtype.
   --   Get/Set_Resolution_Function (Field5)
   --
   --   Get/Set_Resolved_Flag (Flag1)
   --
   --   Get/Set_Signal_Type_Flag (Flag2)

   -- Iir_Kind_File_Type_Definition
   --
   -- True if this is the std.textio.text file type, which may require special
   -- handling.
   --   Get/Set_Text_File_Flag (Flag3)
   --
   --   Get/Set_Type_Declarator (Field1)
   --
   --   Get/Set_Base_Type (Field4)
   --
   --   Get/Set_Type_Mark (Field6)
   --
   --   Get/Set_Resolved_Flag (Flag1)
   --
   --   Get/Set_Signal_Type_Flag (Flag2)

   ---------------------------
   -- concurrent statements --
   ---------------------------

   -- Iir_Kind_Concurrent_Conditional_Signal_Assignment
   -- Iir_Kind_Concurrent_Selected_Signal_Assignment
   --   Get/Set_Label (Ident)
   --
   --   Get/Set_Target (Field2)
   --
   -- True if the target of the assignment is guarded
   --   Get_Guarded_Target_State (State1)
   --
   -- If the assignment is guarded, then get_guard must return the
   -- declaration of the signal guard, otherwise, null_iir.
   -- If the guard signal decl is not known, as a kludge and only to mark this
   -- assignment guarded, the guard can be this assignment.
   --   Get/Set_Guard (Field4)
   --
   --   Get/Set_Delay_Mechanism (Field7)
   --
   --   Get/Set_Reject_Time_Expression (Field5)
   --
   -- For iir_kind_concurrent_selected_signal_assigment only:
   --   Get/Set_Expression (Field3)
   --
   -- For iir_kind_concurrent_selected_signal_assigment only:
   --   Get/Set_Selected_Waveform_Chain (Field1)
   --
   -- For Iir_Kind_Concurrent_Conditional_Signal_Assignment only:
   --   Get/Set_Conditional_Waveform_Chain (Field3)
   --
   --   Get/Set_Postponed_Flag (Flag3)
   --
   --   Get/Set_Attribute_Value_List (Attr)
   --
   --   Get/Set_Chain (Chain)

   -- Iir_Kind_Sensitized_Process_Statement
   -- Iir_Kind_Process_Statement
   --
   --   Get_Declaration_Chain (Field3)
   --
   --   Get/Set_Label (Ident)
   --
   --   Get_Set/Sequential_Statement_Chain (Field2)
   --
   --   Get/Set_Driver_List (Field5)
   --
   --   Get/Set_Is_Within_Flag (Flag5)
   --
   -- Only for iir_kind_sensitized_process_statement
   --   Get_Sensitivity_List (Field4)
   --
   --   Get/Set_Postponed_Flag (Flag3)
   --
   --   Get/Set_Attribute_Value_List (Attr)
   --
   --   Get/Set_Chain (Chain)

   -- Iir_Kind_Concurrent_Assertion_Statement
   --   Get/Set_Label (Ident)
   --
   --   Get/Set_Assertion_Condition (Field2)
   --
   --   Get/Set_Report_Expression (Field3)
   --
   --   Get/Set_Severity_Expression (Field4)
   --
   --   Get/Set_Postponed_Flag (Flag3)
   --
   --   Get/Set_Attribute_Value_List (Attr)
   --
   --   Get/Set_Chain (Chain)

   -- Iir_Kind_Component_Instantiation_Statement
   --   Get/Set_Label (Ident)
   --
   -- Unit instantiated.
   -- Parse: a name, a entity_aspect_entity or a entity_aspect_configuration
   -- Sem: the component declaration or the design unit.
   --   Get/Set_Instantiated_Unit (Field2)
   --
   --   Get/Set_Generic_Map_Aspect_List (Field5)
   --
   --   Get/Set_Port_Map_Aspect_List (Field3)
   --
   -- Configuration:
   -- In case of a configuration specification, the node is put into
   -- default configuration.  In the absence of a specification, the
   -- default entity aspect, if any; if none, this field is null_iir.
   --
   -- During Sem and elaboration, the configuration field can be filled by
   -- a component configuration declaration.
   --
   -- Configuration for this component.
   -- FIXME: must be get/set_binding_indication.
   --   Get/Set_Component_Configuration (Field1)
   --
   -- Default binding indication.
   --   Get/Set_Default_Binding_Indication (Field4)
   --
   --   Get/Set_Attribute_Value_List (Attr)
   --
   --   Get/Set_Chain (Chain)

   -- Iir_Kind_Block_Statement
   --   Get/Set_Label (Ident)
   --
   --   Get/Set_Concurrent_Statement_Chain (Field2)
   --
   --   Get_Declaration_Chain (Field3)
   --
   -- get/set_guard_decl is used for semantic analysis, in order to add
   -- a signal declaration.
   --   Get/Set_Guard_Decl (Field1)
   --
   --   Get/Set_Block_Block_Configuration (Field4)
   --
   --   Get/Set_Block_Header (Field7)
   --
   --   Get/Set_Is_Within_Flag (Flag5)
   --
   --   Get/Set_Attribute_Value_List (Attr)
   --
   --   Get/Set_Chain (Chain)

   -- Iir_Kind_Generate_Statement
   --   Get/Set_Label (Ident)
   --
   -- The generation scheme.
   -- A (boolean) expression for a conditionnal elaboration (if).
   -- A (iterator) declaration for an iterative elaboration (for).
   --   Get/Set_Generation_Scheme (Field5)
   --
   --   Get/Set_Concurrent_Statement_Chain (Field2)
   --
   --   Get_Declaration_Chain (Field3)
   --
   -- The block configuration for this statement.
   --   Get/Set_Generate_Block_Configuration (Field1)
   --
   --   Get/Set_Attribute_Value_List (Attr)
   --
   --   Get/Set_Chain (Chain)

   ---------------------------
   -- sequential statements --
   ---------------------------

   -- Iir_Kind_If_Statement
   -- Iir_Kind_Elsif
   --   Get/Set_Label (Ident)
   --   Get/Set_Parent (Parent)
   --
   -- May be NULL only for an iir_kind_elsif node, and then means the else
   -- clause.
   --   Get/Set_Condition (Field1)
   --
   --   Get/Set_Sequential_Statement_Chain (Field2)
   --
   -- Must be an Iir_kind_elsif node, or NULL for no more elsif clauses.
   --   Get/Set_Else_Clause (Field3)
   --
   -- Only for iir_kind_if_statement
   --   Get/Set_Attribute_Value_List (Attr)

   -- Iir_Kind_For_Loop_Statement
   --   Get/Set_Label (Ident)
   --   Get/Set_Parent (Parent)
   --
   --   Get/Set_Iterator_Scheme (Field1)
   --
   --   Get/Set_Sequential_Statement_Chain (Field2)
   --
   --   Get/Set_Is_Within_Flag (Flag5)
   --
   --   Get/Set_Attribute_Value_List (Attr)

   -- Iir_Kind_While_Loop_Statement
   --   Get/Set_Label (Ident)
   --   Get/Set_Parent (Parent)
   --
   --   Get/Set_Condition (Field1)
   --
   --   Get/Set_Sequential_Statement_Chain (Field2)
   --
   --   Get/Set_Attribute_Value_List (Attr)

   -- Iir_Kind_Exit_Statement
   -- Iir_Kind_Next_Statement
   --   Get/Set_Label (Ident)
   --   Get/Set_Parent (Parent)
   --
   --   Get/Set_Condition (Field1)
   --
   -- Label identifier after parse.
   --   Get/Set_Loop (Field2)
   --
   --   Get/Set_Attribute_Value_List (Attr)

   -- Iir_Kind_Signal_Assignment_Statement
   --   Get/Set_Label (Ident)
   --
   --   Get/Set_Target (Field2)
   --
   -- True if the target of the assignment is guarded
   --   Get_Guarded_Target_State (State1)
   --
   -- The waveform.
   -- If the waveform_chain is null_iir, then the signal assignment is a
   -- disconnection statement, ie TARGET <= null_iir after disconection_time,
   -- where disconnection_time is specified by a disconnection specification.
   --   Get/Set_Waveform_Chain (Field3)
   --
   --   Get/Set_Delay_Mechanism (Field7)
   --
   --   Get/Set_Reject_Time_Expression (Field5)
   --
   --   Get/Set_Attribute_Value_List (Attr)

   -- Iir_Kind_Variable_Assignment_Statement
   --   Get/Set_Label (Ident)
   --
   --   Get/Set_Target (Field2)
   --
   --   Get/Set_Expression (Field3)
   --
   --   Get/Set_Attribute_Value_List (Attr)

   -- Iir_Kind_Assertion_Statement
   --   Get/Set_Label (Ident)
   --
   --   Get/Set_Assertion_Condition (Field2)
   --
   --   Get/Set_Report_Expression (Field3)
   --
   --   Get/Set_Severity_Expression (Field4)
   --
   --   Get/Set_Attribute_Value_List (Attr)

   -- Iir_Kind_Report_Statement
   --   Get/Set_Label (Ident)
   --
   --   Get/Set_Report_Expression (Field3)
   --
   --   Get/Set_Severity_Expression (Field4)
   --
   --   Get/Set_Attribute_Value_List (Attr)

   -- Iir_Kind_Wait_Statement
   --   Get/Set_Label (Ident)
   --
   --   Get/Set_Sensitivity_List (Field4)
   --
   --   Get/Set_Condition_Clause (Field2)
   --
   --   Get/Set_Timeout_Clause (Field3)
   --
   --   Get/Set_Attribute_Value_List (Attr)

   -- Iir_Kind_Return_Statement
   --   Get/Set_Label (Ident)
   --
   -- Type of the return value of the function.  This is a copy of return_type.
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Expression (Field3)
   --
   --   Get/Set_Attribute_Value_List (Attr)

   -- Iir_Kind_Case_Statement
   --   Get/Set_Label (Ident)
   --   Get/Set_Parent (Parent)
   --
   --   Get/Set_Expression (Field3)
   --
   --   Get/Set_Case_Statement_Alternative_Chain (Field4)
   --
   --   Get/Set_Attribute_Value_List (Attr)

   -- Iir_Kind_Procedure_Call
   -- Iir_Kind_Concurrent_Procedure_Call_Statement
   --   Get/Set_Label (Ident)
   --
   --   Get/Set_Implementation (Field3)
   --
   --   Get/Set_Parameter_Association_Chain (Field2)
   --
   -- Only for concurrent_procedure_call_statement
   --   Get/Set_Postponed_Flag (Flag3)
   --
   --   Get/Set_Attribute_Value_List (Attr)

   -- Iir_Kind_Null_Statement
   --   Get/Set_Label (Ident)
   --
   --   Get/Set_Attribute_Value_List (Attr)

   ---------------
   -- operators --
   ---------------

   -- Iir_Kinds_Monadic_Operator
   --   Get/Set_Type (Field1)
   --
   -- Expr_staticness is defined by 7.4
   --   Get/Set_Expr_Staticness (Staticness_1)
   --
   --   Get/Set_Operand (Field2)
   --
   -- Function declaration corresponding to the function to call.
   --   Get/Set_Implementation (Field3)

   -- Iir_Kinds_Dyadic_Operator
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)
   --
   -- Left and Right operands.
   --   Get/Set_Left (Field2)
   --
   --   Get/Set_Right (Field4)
   --
   -- Function declaration corresponding to the function to call.
   --   Get/Set_Implementation (Field3)

   -- Iir_Kind_Function_Call
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Base_Name (Field4)
   --
   -- Function declaration corresponding to the function to call.
   --   Get/Set_Implementation (Field3)
   --
   --   Get/Set_Parameter_Association_Chain (Field2)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)
   --
   --   Get/Set_Name_Staticness (Staticness_2)

   -- Iir_Kind_Aggregate
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Aggregate_Info (Field3)
   --
   --   Get/Set_Association_Choices_Chain (Field4)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)

   -- Iir_Kind_Aggregate_Info
   --
   -- Get info for the next dimension.  NULL_IIR terminated.
   --   Get/Set_Sub_Aggregate_Info (Field1)
   --
   -- For array aggregate only:
   -- If TRUE, the aggregate bounds are not locally static.
   -- This flag is only valid when the array aggregate is constrained, ie
   -- has no 'others' choice.
   --   Get/Set_Aggr_Dynamic_Flag (Flag3)
   --
   -- If TRUE, the aggregate is named, else it is positionnal.
   --   Get/Set_Aggr_Named_Flag (Flag4)
   --
   -- The following three fields are used to check bounds of an array
   -- aggregate.
   -- For named aggregate, low and high bounds are computed, for positionnal
   -- aggregate, the (minimum) number of elements is computed.
   -- Note there may be elements beyond the bounds, due to other choice.
   -- These fields may apply for the aggregate or for the aggregate and its
   -- brothers if the node is for a sub-aggregate.
   --
   -- The low and high index choice, if any.
   --   Get/Set_Aggr_Low_Limit (Field2)
   --
   --   Get/Set_Aggr_High_Limit (Field3)
   --
   -- The maximum number of elements, if any.
   --   Get/Set_Aggr_Max_Length (Nbr2)
   --
   -- True if the choice list has an 'others' choice.
   --   Get/Set_Aggr_Others_Flag (Flag2)

   -- Iir_Kind_Qualified_Expression
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Expression (Field3)
   --
   --   Get/Set_Type_Mark (Field6)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)

   -- Iir_Kind_Type_Conversion
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Expression (Field3)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)

   -- Iir_Kind_Allocator_By_Expression
   -- Iir_Kind_Allocator_By_Subtype
   --
   --   Get/Set_Type (Field1)
   --
   -- Contains the expression for a by expression allocator or the
   -- subtype indication for a by subtype allocator.
   --   Get/Set_Expression (Field3)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)

   -----------
   -- names --
   -----------

   -- Iir_Kind_Simple_Name
   --
   --   Get/Set_Identifier (Ident)
   --
   --   Get/Set_Named_Entity (Field5)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)
   --
   --   Get/Set_Name_Staticness (Staticness_2)

   -- Iir_Kind_Indexed_Name
   -- Select the element designed with the INDEX_LIST from array PREFIX.
   --
   --   Get/Set_Type (Field1)
   --
   --   Get_Index_List (Field2)
   --
   --   Get/Set_Prefix (Field3)
   --
   --   Get/Set_Base_Name (Field4)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)
   --
   --   Get/Set_Name_Staticness (Staticness_2)

   -- Iir_Kind_Slice_Name
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Prefix (Field3)
   --
   --   Get/Set_Suffix (Field2)
   --
   --   Get/Set_Base_Name (Field4)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)
   --
   --   Get/Set_Name_Staticness (Staticness_2)

   -- Iir_Kind_Selected_Name
   --
   --   Get/Set_Identifier (Ident)
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Prefix (Field3)
   --
   --   Get/Set_Named_Entity (Field5)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)
   --
   --   Get/Set_Name_Staticness (Staticness_2)

   -- Iir_Kind_Selected_By_All_Name
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Prefix (Field3)
   --
   --   Get/Set_Base_Name (Field4)
   --
   --   Get/Set_Named_Entity (Field5)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)

   -- Iir_Kind_Parenthesis_Name
   -- Created by the parser, and mutated into the correct iir node: it can be
   -- either a function call, an indexed array, a type conversion or a slice
   -- name.
   --
   -- Always returns null_iir.
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Prefix (Field3)
   --
   --   Get/Set_Named_Entity (Field5)
   --
   --   Get/Set_Association_Chain (Field4)

   ----------------
   -- attributes --
   ----------------

   -- Iir_Kind_Attribute_Name
   --
   --   Get/Set_Identifier (Ident)
   --
   --   Get/Set_Prefix (Field3)
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Named_Entity (Field5)
   --
   --   Get/Set_Signature (Field4)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)

   -- Iir_Kind_Base_Attribute
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Prefix (Field3)

   -- Iir_Kind_Left_Type_Attribute
   -- Iir_Kind_Right_Type_Attribute
   -- Iir_Kind_High_Type_Attribute
   -- Iir_Kind_Low_Type_Attribute
   -- Iir_Kind_Ascending_Type_Attribute
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Prefix (Field3)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)

   -- Iir_Kind_Range_Array_Attribute
   -- Iir_Kind_Reverse_Range_Array_Attribute
   -- Iir_Kind_Left_Array_Attribute
   -- Iir_Kind_Right_Array_Attribute
   -- Iir_Kind_High_Array_Attribute
   -- Iir_Kind_Low_Array_Attribute
   -- Iir_Kind_Ascending_Array_Attribute
   -- Iir_Kind_Length_Array_Attribute
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Prefix (Field3)
   --
   --   Get/Set_Suffix (Field2)
   --
   --   Get/Set_Base_Type (Field4)
   --
   --   Get/Set_Type_Staticness (Staticness_1)

   -- Iir_Kind_Stable_Attribute
   -- Iir_Kind_Delayed_Attribute
   -- Iir_Kind_Quiet_Attribute
   -- Iir_Kind_Transaction_Attribute
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Prefix (Field3)
   --
   --   Get/Set_Parameter (Field2)
   --
   --   Get/Set_Base_Type (Field4)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)
   --
   --   Get/Set_Name_Staticness (Staticness_2)

   -- Iir_Kind_Event_Attribute
   -- Iir_Kind_Last_Event_Attribute
   -- Iir_Kind_Last_Value_Attribute
   -- Iir_Kind_Active_Attribute
   -- Iir_Kind_Last_Active_Attribute
   -- Iir_Kind_Driving_Attribute
   -- Iir_Kind_Driving_Value_Attribute
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Prefix (Field3)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)
   --
   --   Get/Set_Name_Staticness (Staticness_2)

   -- Iir_Kind_Pos_Attribute
   -- Iir_Kind_Val_Attribute
   -- Iir_Kind_Succ_Attribute
   -- Iir_Kind_Pred_Attribute
   -- Iir_Kind_Leftof_Attribute
   -- Iir_Kind_Rightof_Attribute
   --
   -- type/subtype declaration.
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Prefix (Field3)
   --
   --   Get/Set_Parameter (Field2)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)

   -- Iir_Kind_Image_Attribute
   -- Iir_Kind_Value_Attribute
   --
   --   Get/Set_Type (Field1)
   --
   --   Get/Set_Prefix (Field3)
   --
   --   Get/Set_Parameter (Field2)
   --
   --   Get/Set_Expr_Staticness (Staticness_1)

   -- Iir_Kind_Simple_Name_Attribute
   -- Iir_Kind_Instance_Name_Attribute
   -- Iir_Kind_Path_Name_Attribute
   -- FIXME: to describe

   -- Iir_Kind_Behavior_Attribute
   -- Iir_Kind_Structure_Attribute
   -- FIXME: to describe

   -- Iir_Kind_Error

   -- End of Iir_Kind.


   type Iir_Kind is
      (
       Iir_Kind_Design_File,
       Iir_Kind_Design_Unit,
       Iir_Kind_Library_Clause,
       Iir_Kind_Use_Clause,

   -- Literals.
       Iir_Kind_Character_Literal,
       Iir_Kind_Integer_Literal,
       Iir_Kind_Floating_Point_Literal,
       Iir_Kind_Null_Literal,
       Iir_Kind_String_Literal,
       Iir_Kind_Physical_Int_Literal,
       Iir_Kind_Physical_Fp_Literal,
       Iir_Kind_Bit_String_Literal,
       Iir_Kind_Simple_Aggregate,

   -- Tuple,
       Iir_Kind_Proxy,
       Iir_Kind_Waveform_Element,
       Iir_Kind_Conditional_Waveform,
       Iir_Kind_Association_Element_By_Expression,
       Iir_Kind_Association_Element_By_Individual,
       Iir_Kind_Association_Element_Open,
       Iir_Kind_Choice_By_Others,
       Iir_Kind_Choice_By_Expression,
       Iir_Kind_Choice_By_Range,
       Iir_Kind_Choice_By_None,
       Iir_Kind_Choice_By_Name,
       Iir_Kind_Designator_By_All,
       Iir_Kind_Designator_By_Others,
       Iir_Kind_Entity_Aspect_Entity,
       Iir_Kind_Entity_Aspect_Configuration,
       Iir_Kind_Entity_Aspect_Open,
       Iir_Kind_Block_Configuration,
       Iir_Kind_Block_Header,
       Iir_Kind_Component_Configuration,
       Iir_Kind_Binding_Indication,
       Iir_Kind_Entity_Class,
       Iir_Kind_Attribute_Value,
       Iir_Kind_Signature,
       Iir_Kind_Aggregate_Info,

       Iir_Kind_Attribute_Specification,
       Iir_Kind_Disconnection_Specification,
       Iir_Kind_Configuration_Specification,

   -- Type definitions.
   -- iir_kinds_type_and_subtype_definition
   -- kinds: disc: discrete, st: subtype.
       Iir_Kind_Access_Type_Definition,
       Iir_Kind_Incomplete_Type_Definition,
       Iir_Kind_File_Type_Definition,
       Iir_Kind_Record_Type_Definition,                 -- composite
       Iir_Kind_Array_Type_Definition,                  -- composite, array
       Iir_Kind_Unconstrained_Array_Subtype_Definition, -- composite, array, st
       Iir_Kind_Array_Subtype_Definition,               -- composite, array, st
       Iir_Kind_Record_Subtype_Definition,              -- composite, st
       Iir_Kind_Access_Subtype_Definition,              -- st
       Iir_Kind_Physical_Subtype_Definition,            -- scalar, st
       Iir_Kind_Floating_Subtype_Definition,            -- scalar, st
       Iir_Kind_Integer_Subtype_Definition,             -- scalar, disc, st
       Iir_Kind_Enumeration_Subtype_Definition,         -- scalar, disc, st
       Iir_Kind_Integer_Type_Definition,                -- scalar, disc
       Iir_Kind_Enumeration_Type_Definition,            -- scalar, disc
       Iir_Kind_Floating_Type_Definition,               -- scalar
       Iir_Kind_Physical_Type_Definition,               -- scalar
       Iir_Kind_Range_Expression,
       Iir_Kind_Subtype_Definition,  -- temporary (must not appear after sem).

   -- Lists.
       Iir_Kind_List,
       Iir_Kind_Index_List,
       Iir_Kind_Design_Unit_List,
       Iir_Kind_Enumeration_Literal_List,
       Iir_Kind_Designator_List,
       Iir_Kind_Overload_List,  -- used internally by sem_expr.
       Iir_Kind_Driver_List,
       Iir_Kind_Attribute_Value_List,
       Iir_Kind_Group_Constituent_List,
       Iir_Kind_Callees_List,

   -- Declarations.
   --  iir_kinds_nonoverloadable_declaration
       Iir_Kind_Type_Declaration,
       Iir_Kind_Anonymous_Type_Declaration,
       Iir_Kind_Subtype_Declaration,
       Iir_Kind_Configuration_Declaration,
       Iir_Kind_Entity_Declaration,
       Iir_Kind_Package_Declaration,
       Iir_Kind_Package_Body,
       Iir_Kind_Architecture_Declaration,
       Iir_Kind_Unit_Declaration,
       Iir_Kind_Library_Declaration,
       Iir_Kind_Component_Declaration,
       Iir_Kind_Attribute_Declaration,
       Iir_Kind_Group_Template_Declaration,
       Iir_Kind_Group_Declaration,
       Iir_Kind_Element_Declaration,
       Iir_Kind_Non_Object_Alias_Declaration,

       Iir_Kind_Function_Body,                  -- f_decl, f_body
       Iir_Kind_Function_Declaration,           -- f_decl, f_spec, ufs
       Iir_Kind_Implicit_Function_Declaration,  -- f_decl, f_spec
       Iir_Kind_Implicit_Procedure_Declaration, -- p_decl, p_spec
       Iir_Kind_Procedure_Declaration,          -- p_decl, p_spec, ups
       Iir_Kind_Procedure_Body,                 -- p_decl, p_body
       Iir_Kind_Enumeration_Literal,

       Iir_Kind_File_Declaration,               -- object
       Iir_Kind_Object_Alias_Declaration,       -- object
       Iir_Kind_Guard_Signal_Declaration,       -- object
       Iir_Kind_Signal_Declaration,             -- object
       Iir_Kind_Variable_Declaration,           -- object
       Iir_Kind_Constant_Declaration,           -- object
       Iir_Kind_Iterator_Declaration,           -- object
       Iir_Kind_Constant_Interface_Declaration, -- object, interface
       Iir_Kind_Variable_Interface_Declaration, -- object, interface
       Iir_Kind_Signal_Interface_Declaration,   -- object, interface
       Iir_Kind_File_Interface_Declaration,     -- object, interface

   -- Expressions.
       Iir_Kind_Identity_Operator,
       Iir_Kind_Negation_Operator,
       Iir_Kind_Absolute_Operator,
       Iir_Kind_Not_Operator,
       Iir_Kind_And_Operator,
       Iir_Kind_Or_Operator,
       Iir_Kind_Nand_Operator,
       Iir_Kind_Nor_Operator,
       Iir_Kind_Xor_Operator,
       Iir_Kind_Xnor_Operator,
       Iir_Kind_Equality_Operator,
       Iir_Kind_Inequality_Operator,
       Iir_Kind_Less_Than_Operator,
       Iir_Kind_Less_Than_Or_Equal_Operator,
       Iir_Kind_Greater_Than_Operator,
       Iir_Kind_Greater_Than_Or_Equal_Operator,
       Iir_Kind_Sll_Operator,
       Iir_Kind_Sla_Operator,
       Iir_Kind_Srl_Operator,
       Iir_Kind_Sra_Operator,
       Iir_Kind_Rol_Operator,
       Iir_Kind_Ror_Operator,
       Iir_Kind_Addition_Operator,
       Iir_Kind_Substraction_Operator,
       Iir_Kind_Concatenation_Operator,
       Iir_Kind_Multiplication_Operator,
       Iir_Kind_Division_Operator,
       Iir_Kind_Modulus_Operator,
       Iir_Kind_Remainder_Operator,
       Iir_Kind_Exponentiation_Operator,
       Iir_Kind_Function_Call,
       Iir_Kind_Aggregate,
       Iir_Kind_Qualified_Expression,
       Iir_Kind_Type_Conversion,
       Iir_Kind_Allocator_By_Expression,
       Iir_Kind_Allocator_By_Subtype,
       Iir_Kind_Selected_Element,
       Iir_Kind_Dereference,
       Iir_Kind_Implicit_Dereference,

   -- Concurrent statements.
       Iir_Kind_Sensitized_Process_Statement,
       Iir_Kind_Process_Statement,
       Iir_Kind_Concurrent_Conditional_Signal_Assignment,
       Iir_Kind_Concurrent_Selected_Signal_Assignment,
       Iir_Kind_Concurrent_Assertion_Statement,
       Iir_Kind_Concurrent_Procedure_Call_Statement,
       Iir_Kind_Block_Statement,
       Iir_Kind_Generate_Statement,
       Iir_Kind_Component_Instantiation_Statement,

   -- Iir_Kind_Sequential_Statement
       Iir_Kind_Signal_Assignment_Statement,
       Iir_Kind_Null_Statement,
       Iir_Kind_Assertion_Statement,
       Iir_Kind_Report_Statement,
       Iir_Kind_Wait_Statement,
       Iir_Kind_Variable_Assignment_Statement,
       Iir_Kind_Return_Statement,
       Iir_Kind_For_Loop_Statement,
       Iir_Kind_While_Loop_Statement,
       Iir_Kind_Next_Statement,
       Iir_Kind_Exit_Statement,
       Iir_Kind_Case_Statement,
       Iir_Kind_Procedure_Call,
       Iir_Kind_If_Statement,
       Iir_Kind_Elsif,

   -- Names
       Iir_Kind_Simple_Name,
       Iir_Kind_Slice_Name,
       Iir_Kind_Indexed_Name,
       Iir_Kind_Selected_Name,
       Iir_Kind_Selected_By_All_Name,
       Iir_Kind_Parenthesis_Name,

   -- Attributes
       Iir_Kind_Base_Attribute,
       Iir_Kind_Left_Type_Attribute,            --  type_attribute
       Iir_Kind_Right_Type_Attribute,           --  type_attribute
       Iir_Kind_High_Type_Attribute,            --  type_attribute
       Iir_Kind_Low_Type_Attribute,             --  type_attribute
       Iir_Kind_Ascending_Type_Attribute,       --  type_attribute
       Iir_Kind_Image_Attribute,
       Iir_Kind_Value_Attribute,
       Iir_Kind_Pos_Attribute,                  --  scalar_type_attribute
       Iir_Kind_Val_Attribute,                  --  scalar_type_attribute
       Iir_Kind_Succ_Attribute,                 --  scalar_type_attribute
       Iir_Kind_Pred_Attribute,                 --  scalar_type_attribute
       Iir_Kind_Leftof_Attribute,               --  scalar_type_attribute
       Iir_Kind_Rightof_Attribute,              --  scalar_type_attribute
       Iir_Kind_Left_Array_Attribute,           --  array_attribute
       Iir_Kind_Right_Array_Attribute,          --  array_attribute
       Iir_Kind_High_Array_Attribute,           --  array_attribute
       Iir_Kind_Low_Array_Attribute,            --  array_attribute
       Iir_Kind_Range_Array_Attribute,          --  array_attribute
       Iir_Kind_Reverse_Range_Array_Attribute,  --  array_attribute
       Iir_Kind_Length_Array_Attribute,         --  array_attribute
       Iir_Kind_Ascending_Array_Attribute,      --  array_attribute
       Iir_Kind_Delayed_Attribute,              --  signal_attribute
       Iir_Kind_Stable_Attribute,               --  signal_attribute
       Iir_Kind_Quiet_Attribute,                --  signal_attribute
       Iir_Kind_Transaction_Attribute,          --  signal_attribute
       Iir_Kind_Event_Attribute,                --  signal_value_attribute
       Iir_Kind_Active_Attribute,               --  signal_value_attribute
       Iir_Kind_Last_Event_Attribute,           --  signal_value_attribute
       Iir_Kind_Last_Active_Attribute,          --  signal_value_attribute
       Iir_Kind_Last_Value_Attribute,           --  signal_value_attribute
       Iir_Kind_Driving_Attribute,              --  signal_value_attribute
       Iir_Kind_Driving_Value_Attribute,        --  signal_value_attribute
       Iir_Kind_Behavior_Attribute,
       Iir_Kind_Structure_Attribute,
       Iir_Kind_Simple_Name_Attribute,
       Iir_Kind_Instance_Name_Attribute,
       Iir_Kind_Path_Name_Attribute,

       Iir_Kind_Attribute_Name,

   -- Erroneous IIR.
       Iir_Kind_Error
      );

   type Iir_Signal_Kind is
      (
       Iir_No_Signal_Kind,
       Iir_Register_Kind,
       Iir_Bus_Kind
       );

   type Iir_Mode is
      (
       Iir_Unknown_Mode,
       Iir_In_Mode,
       Iir_Out_Mode,
       Iir_Inout_Mode,
       Iir_Buffer_Mode,
       Iir_Linkage_Mode
       );

   type Iir_Delay_Mechanism is (Iir_Inertial_Delay, Iir_Transport_Delay);

   type Iir_Direction is (Iir_To, Iir_Downto);

   --  Iir_Lexical_Layout_type describe the lexical token used to describe
   --  an interface declaration.  This has no semantics meaning, but it is
   --  necessary to keep how lexically an interface was declared due to
   --  LRM93 2.7 (conformance rules).
   --  To keep this simple, the layout is stored as a bit-string.
   --  Fields are:
   --  Has_type: set if the interface is the last of a list.
   --  has_mode: set if mode is explicit
   --  has_class: set if class (constant, signal, variable or file) is explicit
   --
   --  Exemple:
   --  procedure P (A,B: integer;
   --               C: in constant bit;
   --               D: inout bit;
   --               E: variable bit;
   --               F, G: in bit;
   --               H, I: constant bit;
   --               J, K: in constant bit);
   --  A:
   --  B:                         has_type
   --  C, K: has_mode, has_class, has_type
   --  D:    has_mode,            has_type
   --  E, I:           has_class, has_type
   --  F:    has_mode
   --  G:    has_mode,            has_type
   --  H:              has_class
   --  J:    has_mode, has_class
   type Iir_Lexical_Layout_Type is mod 2 ** 3;
   Iir_Lexical_Has_Mode  : constant Iir_Lexical_Layout_Type := 2 ** 0;
   Iir_Lexical_Has_Class : constant Iir_Lexical_Layout_Type := 2 ** 1;
   Iir_Lexical_Has_Type  : constant Iir_Lexical_Layout_Type := 2 ** 2;

   --  List of predefined operators and functions.
   type Iir_Predefined_Functions is
      (
       Iir_Predefined_Error,

   --  Predefined operators for BOOLEAN type.
       Iir_Predefined_Boolean_And,
       Iir_Predefined_Boolean_Or,
       Iir_Predefined_Boolean_Nand,
       Iir_Predefined_Boolean_Nor,
       Iir_Predefined_Boolean_Xor,
       Iir_Predefined_Boolean_Xnor,
       Iir_Predefined_Boolean_Not,

   --  Predefined operators for any enumeration type.
       Iir_Predefined_Enum_Equality,
       Iir_Predefined_Enum_Inequality,
       Iir_Predefined_Enum_Less,
       Iir_Predefined_Enum_Less_Equal,
       Iir_Predefined_Enum_Greater,
       Iir_Predefined_Enum_Greater_Equal,

   --  Predefined operators for BIT type.
       Iir_Predefined_Bit_And,
       Iir_Predefined_Bit_Or,
       Iir_Predefined_Bit_Nand,
       Iir_Predefined_Bit_Nor,
       Iir_Predefined_Bit_Xor,
       Iir_Predefined_Bit_Xnor,
       Iir_Predefined_Bit_Not,

   --  Predefined operators for any integer type.
       Iir_Predefined_Integer_Equality,
       Iir_Predefined_Integer_Inequality,
       Iir_Predefined_Integer_Less,
       Iir_Predefined_Integer_Less_Equal,
       Iir_Predefined_Integer_Greater,
       Iir_Predefined_Integer_Greater_Equal,

       Iir_Predefined_Integer_Identity,
       Iir_Predefined_Integer_Negation,
       Iir_Predefined_Integer_Absolute,

       Iir_Predefined_Integer_Plus,
       Iir_Predefined_Integer_Minus,
       Iir_Predefined_Integer_Mul,
       Iir_Predefined_Integer_Div,
       Iir_Predefined_Integer_Mod,
       Iir_Predefined_Integer_Rem,

       Iir_Predefined_Integer_Exp,

   --  Predefined operators for any floating type.
       Iir_Predefined_Floating_Equality,
       Iir_Predefined_Floating_Inequality,
       Iir_Predefined_Floating_Less,
       Iir_Predefined_Floating_Less_Equal,
       Iir_Predefined_Floating_Greater,
       Iir_Predefined_Floating_Greater_Equal,

       Iir_Predefined_Floating_Identity,
       Iir_Predefined_Floating_Negation,
       Iir_Predefined_Floating_Absolute,

       Iir_Predefined_Floating_Plus,
       Iir_Predefined_Floating_Minus,
       Iir_Predefined_Floating_Mul,
       Iir_Predefined_Floating_Div,
       Iir_Predefined_Floating_Mod,
       Iir_Predefined_Floating_Rem,

       Iir_Predefined_Floating_Exp,

   --  Predefined operator for universal types.
       Iir_Predefined_Universal_R_I_Mul,
       Iir_Predefined_Universal_I_R_Mul,
       Iir_Predefined_Universal_R_I_Div,

   --  Predefined operators for physical types.
       Iir_Predefined_Physical_Equality,
       Iir_Predefined_Physical_Inequality,
       Iir_Predefined_Physical_Less,
       Iir_Predefined_Physical_Less_Equal,
       Iir_Predefined_Physical_Greater,
       Iir_Predefined_Physical_Greater_Equal,

       Iir_Predefined_Physical_Identity,
       Iir_Predefined_Physical_Negation,
       Iir_Predefined_Physical_Absolute,

       Iir_Predefined_Physical_Plus,
       Iir_Predefined_Physical_Minus,

       Iir_Predefined_Physical_Integer_Mul,
       Iir_Predefined_Physical_Real_Mul,
       Iir_Predefined_Integer_Physical_Mul,
       Iir_Predefined_Real_Physical_Mul,
       Iir_Predefined_Physical_Integer_Div,
       Iir_Predefined_Physical_Real_Div,
       Iir_Predefined_Physical_Physical_Div,

   --  Predefined operators for access.
       Iir_Predefined_Access_Equality,
       Iir_Predefined_Access_Inequality,

   --  Predefined operators for record.
       Iir_Predefined_Record_Equality,
       Iir_Predefined_Record_Inequality,

   --  Predefined operators for array.
       Iir_Predefined_Array_Equality,
       Iir_Predefined_Array_Inequality,
       Iir_Predefined_Array_Less,
       Iir_Predefined_Array_Less_Equal,
       Iir_Predefined_Array_Greater,
       Iir_Predefined_Array_Greater_Equal,

       Iir_Predefined_Array_Array_Concat,
       Iir_Predefined_Array_Element_Concat,
       Iir_Predefined_Element_Array_Concat,
       Iir_Predefined_Element_Element_Concat,

   --  Predefined shift operators.
       Iir_Predefined_Array_Sll,
       Iir_Predefined_Array_Srl,
       Iir_Predefined_Array_Sla,
       Iir_Predefined_Array_Sra,
       Iir_Predefined_Array_Rol,
       Iir_Predefined_Array_Ror,

   --  Predefined operators for one dimensional array
       Iir_Predefined_Bit_Array_And,
       Iir_Predefined_Bit_Array_Or,
       Iir_Predefined_Bit_Array_Nand,
       Iir_Predefined_Bit_Array_Nor,
       Iir_Predefined_Bit_Array_Xor,
       Iir_Predefined_Bit_Array_Xnor,
       Iir_Predefined_Bit_Array_Not,

       Iir_Predefined_Boolean_Array_And,
       Iir_Predefined_Boolean_Array_Or,
       Iir_Predefined_Boolean_Array_Nand,
       Iir_Predefined_Boolean_Array_Nor,
       Iir_Predefined_Boolean_Array_Xor,
       Iir_Predefined_Boolean_Array_Xnor,
       Iir_Predefined_Boolean_Array_Not,

   --  Predefined attribute functions.
       Iir_Predefined_Attribute_Image,
       Iir_Predefined_Attribute_Value,
       Iir_Predefined_Attribute_Pos,
       Iir_Predefined_Attribute_Val,
       Iir_Predefined_Attribute_Succ,
       Iir_Predefined_Attribute_Pred,
       Iir_Predefined_Attribute_Leftof,
       Iir_Predefined_Attribute_Rightof,
       Iir_Predefined_Attribute_Left,
       Iir_Predefined_Attribute_Right,
       Iir_Predefined_Attribute_Low,
       Iir_Predefined_Attribute_Event,
       Iir_Predefined_Attribute_Active,
       Iir_Predefined_Attribute_Last_Event,
       Iir_Predefined_Attribute_Last_Active,
       Iir_Predefined_Attribute_Last_Value,
       Iir_Predefined_Attribute_Driving,
       Iir_Predefined_Attribute_Driving_Value,

   --  Access procedure
       Iir_Predefined_Deallocate,

   --  file function / procedures.
       Iir_Predefined_File_Open,
       Iir_Predefined_File_Open_Status,
       Iir_Predefined_File_Close,
       Iir_Predefined_Read,
       Iir_Predefined_Read_Length,
       Iir_Predefined_Write,
       Iir_Predefined_Endfile,

   --  Predefined function.
       Iir_Predefined_Now_Function
       );

   --  Return TRUE iff FUNC is a short-cut predefined function.
   function Iir_Predefined_Shortcut_P (Func : Iir_Predefined_Functions)
     return Boolean;

   subtype Iir_Predefined_Pure_Functions is Iir_Predefined_Functions range
     Iir_Predefined_Boolean_And .. Iir_Predefined_Attribute_Driving_Value;

   subtype Iir_Predefined_Dyadic_Bit_Array_Functions
   is Iir_Predefined_Functions range
     Iir_Predefined_Bit_Array_And ..
   --Iir_Predefined_Bit_Array_Or
   --Iir_Predefined_Bit_Array_Nand
   --Iir_Predefined_Bit_Array_Nor
   --Iir_Predefined_Bit_Array_Xor
     Iir_Predefined_Bit_Array_Xnor;

   subtype Iir_Predefined_Shift_Functions is Iir_Predefined_Functions range
     Iir_Predefined_Array_Sll ..
   --Iir_Predefined_Array_Srl
   --Iir_Predefined_Array_Sla
   --Iir_Predefined_Array_Sra
   --Iir_Predefined_Array_Rol
     Iir_Predefined_Array_Ror;

   subtype Iir_Predefined_Concat_Functions is Iir_Predefined_Functions range
     Iir_Predefined_Array_Array_Concat ..
   --Iir_Predefined_Array_Element_Concat
   --Iir_Predefined_Element_Array_Concat
     Iir_Predefined_Element_Element_Concat;

   -- Staticness as defined by LRM93 6.1 and 7.4
   type Iir_Staticness is (Unknown, None, Globally, Locally);
   function Min (L,R: Iir_Staticness) return Iir_Staticness renames
     Iir_Staticness'Min;

   ---------------
   -- subranges --
   ---------------
   -- These subtypes are used for ranges, for `case' statments or for the `in'
   -- operator.

   -- In order to be correctly parsed by check_iir, the declaration must
   -- follow these rules:
   -- * the first line must be "subtype Iir_Kinds_NAME is Iir_Kind_range"
   -- * the second line must be the lowest bound of the range, followed by "..
   -- * comments line
   -- * the last line must be the highest bound of the range, followed by ";"

   subtype Iir_Kinds_List is Iir_Kind range
     Iir_Kind_List ..
     Iir_Kind_Callees_List;

   subtype Iir_Kinds_Library_Unit_Declaration is Iir_Kind range
     Iir_Kind_Configuration_Declaration ..
   --Iir_Kind_Entity_Declaration
   --Iir_Kind_Package_Declaration
   --Iir_Kind_Package_Body
     Iir_Kind_Architecture_Declaration;

   --  Note: does not include iir_kind_enumeration_literal since it is
   --  considered as a declaration.
   subtype Iir_Kinds_Literal is Iir_Kind range
     Iir_Kind_Character_Literal ..
   --Iir_Kind_Integer_Literal
   --Iir_Kind_Floating_Point_Literal
   --Iir_Kind_Null_Literal
   --Iir_Kind_String_Literal
   --Iir_Kind_Physical_Int_Literal
   --Iir_Kind_Physical_Fp_Literal
     Iir_Kind_Bit_String_Literal;

   subtype Iir_Kinds_Array_Type_Definition is Iir_Kind range
     Iir_Kind_Array_Type_Definition ..
   --Iir_Kind_Unconstrained_Array_Subtype_Definition
     Iir_Kind_Array_Subtype_Definition;

   subtype Iir_Kinds_Type_And_Subtype_Definition is Iir_Kind range
     Iir_Kind_Access_Type_Definition ..
   --Iir_Kind_Incomplete_Type_Definition
   --Iir_Kind_File_Type_Definition
   --Iir_Kind_Record_Type_Definition
   --Iir_Kind_Array_Type_Definition
   --Iir_Kind_Unconstrained_Array_Subtype_Definition
   --Iir_Kind_Array_Subtype_Definition
   --Iir_Kind_Record_Subtype_Definition
   --Iir_Kind_Access_Subtype_Definition
   --Iir_Kind_Physical_Subtype_Definition
   --Iir_Kind_Floating_Subtype_Definition
   --Iir_Kind_Integer_Subtype_Definition
   --Iir_Kind_Enumeration_Subtype_Definition
   --Iir_Kind_Integer_Type_Definition
   --Iir_Kind_Enumeration_Type_Definition
   --Iir_Kind_Floating_Type_Definition
     Iir_Kind_Physical_Type_Definition;

   subtype Iir_Kinds_Subtype_Definition is Iir_Kind range
     Iir_Kind_Unconstrained_Array_Subtype_Definition ..
   --Iir_Kind_Array_Subtype_Definition
   --Iir_Kind_Record_Subtype_Definition
   --Iir_Kind_Access_Subtype_Definition
   --Iir_Kind_Physical_Subtype_Definition
   --Iir_Kind_Floating_Subtype_Definition
   --Iir_Kind_Integer_Subtype_Definition
     Iir_Kind_Enumeration_Subtype_Definition;

   subtype Iir_Kinds_Scalar_Type_Definition is Iir_Kind range
     Iir_Kind_Physical_Subtype_Definition ..
   --Iir_Kind_Floating_Subtype_Definition
   --Iir_Kind_Integer_Subtype_Definition
   --Iir_Kind_Enumeration_Subtype_Definition
   --Iir_Kind_Integer_Type_Definition
   --Iir_Kind_Enumeration_Type_Definition
   --Iir_Kind_Floating_Type_Definition
     Iir_Kind_Physical_Type_Definition;

   subtype Iir_Kinds_Discrete_Type_Definition is Iir_Kind range
     Iir_Kind_Integer_Subtype_Definition ..
   --Iir_Kind_Enumeration_Subtype_Definition
   --Iir_Kind_Integer_Type_Definition
     Iir_Kind_Enumeration_Type_Definition;

   subtype Iir_Kinds_Composite_Type_Definition is Iir_Kind range
     Iir_Kind_Record_Type_Definition ..
   --Iir_Kind_Array_Type_Definition
   --Iir_Kind_Unconstrained_Array_Subtype_Definition
   --Iir_Kind_Array_Subtype_Definition
     Iir_Kind_Record_Subtype_Definition;

   subtype Iir_Kinds_Unconstrained_Array_Type_Definition is Iir_Kind range
     Iir_Kind_Array_Type_Definition ..
     Iir_Kind_Unconstrained_Array_Subtype_Definition;

   subtype Iir_Kinds_Array_Subtype_Definition is Iir_Kind range
     Iir_Kind_Unconstrained_Array_Subtype_Definition ..
     Iir_Kind_Array_Subtype_Definition;

   subtype Iir_Kinds_Type_Declaration is Iir_Kind range
     Iir_Kind_Type_Declaration ..
   --Iir_Kind_Anonymous_Type_Declaration
     Iir_Kind_Subtype_Declaration;

   subtype Iir_Kinds_Nonoverloadable_Declaration is Iir_Kind range
     Iir_Kind_Type_Declaration ..
     Iir_Kind_Element_Declaration;

   subtype Iir_Kinds_Monadic_Operator is Iir_Kind range
     Iir_Kind_Identity_Operator ..
   --Iir_Kind_Negation_Operator
   --Iir_Kind_Absolute_Operator
     Iir_Kind_Not_Operator;

   subtype Iir_Kinds_Dyadic_Operator is Iir_Kind range
     Iir_Kind_And_Operator ..
   --Iir_Kind_Or_Operator
   --Iir_Kind_Nand_Operator
   --Iir_Kind_Nor_Operator
   --Iir_Kind_Xor_Operator
   --Iir_Kind_Xnor_Operator
   --Iir_Kind_Equality_Operator
   --Iir_Kind_Inequality_Operator
   --Iir_Kind_Less_Than_Operator
   --Iir_Kind_Less_Than_Or_Equal_Operator
   --Iir_Kind_Greater_Than_Operator
   --Iir_Kind_Greater_Than_Or_Equal_Operator
   --Iir_Kind_Sll_Operator
   --Iir_Kind_Sla_Operator
   --Iir_Kind_Srl_Operator
   --Iir_Kind_Sra_Operator
   --Iir_Kind_Rol_Operator
   --Iir_Kind_Ror_Operator
   --Iir_Kind_Addition_Operator
   --Iir_Kind_Substraction_Operator
   --Iir_Kind_Concatenation_Operator
   --Iir_Kind_Multiplication_Operator
   --Iir_Kind_Division_Operator
   --Iir_Kind_Modulus_Operator
   --Iir_Kind_Remainder_Operator
     Iir_Kind_Exponentiation_Operator;

--    subtype Iir_Kinds_Subprogram_Declaration is Iir_Kind range
--      Iir_Kind_Implicit_Function_Declaration ..
--    --Iir_Kind_Function_Specification
--    --Iir_Kind_Function_Spec_Body
--    --Iir_Kind_Function_Body
--    --Iir_Kind_Implicit_Procedure_Declaration
--    --Iir_Kind_Procedure_Specification
--    --Iir_Kind_Procedure_Spec_Body
--      Iir_Kind_Procedure_Body;

   subtype Iir_Kinds_Function_Declaration is Iir_Kind range
     Iir_Kind_Function_Declaration ..
     Iir_Kind_Implicit_Function_Declaration;

--    subtype Iir_Kinds_Function_Specification is Iir_Kind range
--      Iir_Kind_Implicit_Function_Declaration ..
--    --Iir_Kind_Function_Specification
--      Iir_Kind_Function_Spec_Body;

--    subtype Iir_Kinds_User_Function_Specification is Iir_Kind range
--      Iir_Kind_Function_Specification ..
--      Iir_Kind_Function_Spec_Body;

--    subtype Iir_Kinds_Function_Body is Iir_Kind range
--      Iir_Kind_Function_Spec_Body ..
--      Iir_Kind_Function_Body;

   subtype Iir_Kinds_Procedure_Declaration is Iir_Kind range
      Iir_Kind_Implicit_Procedure_Declaration ..
     Iir_Kind_Procedure_Declaration;

--    subtype Iir_Kinds_Procedure_Specification is Iir_Kind range
--      Iir_Kind_Implicit_Procedure_Declaration ..
--    --Iir_Kind_Procedure_Specification
--      Iir_Kind_Procedure_Spec_Body;

--    subtype Iir_Kinds_User_Procedure_Specification is Iir_Kind range
--      Iir_Kind_Procedure_Specification ..
--      Iir_Kind_Procedure_Spec_Body;

--    subtype Iir_Kinds_Procedure_Body is Iir_Kind range
--      Iir_Kind_Procedure_Spec_Body ..
--      Iir_Kind_Procedure_Body;

   subtype Iir_Kinds_Process_Statement is Iir_Kind range
     Iir_Kind_Sensitized_Process_Statement ..
     Iir_Kind_Process_Statement;

   subtype Iir_Kinds_Interface_Declaration is Iir_Kind range
     Iir_Kind_Constant_Interface_Declaration ..
   --Iir_Kind_Variable_Interface_Declaration
   --Iir_Kind_Signal_Interface_Declaration
     Iir_Kind_File_Interface_Declaration;

   subtype Iir_Kinds_Object_Declaration is Iir_Kind range
     Iir_Kind_File_Declaration ..
   --Iir_Kind_Object_Alias_Declaration
   --Iir_Kind_Guard_Signal_Declaration
   --Iir_Kind_Signal_Declaration
   --Iir_Kind_Variable_Declaration
   --Iir_Kind_Constant_Declaration
   --Iir_Kind_Iterator_Declaration
   --Iir_Kind_Constant_Interface_Declaration
   --Iir_Kind_Variable_Interface_Declaration
   --Iir_Kind_Signal_Interface_Declaration
     Iir_Kind_File_Interface_Declaration;

   subtype Iir_Kinds_Association_Element is Iir_Kind range
     Iir_Kind_Association_Element_By_Expression ..
   --Iir_Kind_Association_Element_By_Individual
     Iir_Kind_Association_Element_Open;

   subtype Iir_Kinds_Choice is Iir_Kind range
     Iir_Kind_Choice_By_Others ..
   --Iir_Kind_Choice_By_Expression
   --Iir_Kind_Choice_By_Range
   --Iir_Kind_Choice_By_None
     Iir_Kind_Choice_By_Name;

   subtype Iir_Kinds_Name is Iir_Kind range
     Iir_Kind_Simple_Name ..
   --Iir_Kind_Slice_Name
   --Iir_Kind_Indexed_Name
   --Iir_Kind_Selected_Name
   --Iir_Kind_Selected_By_All_Name
     Iir_Kind_Parenthesis_Name;

   --  Any attribute that is an expression.
   subtype Iir_Kinds_Expression_Attribute is Iir_Kind range
     Iir_Kind_Left_Type_Attribute ..
   --Iir_Kind_Right_Type_Attribute
   --Iir_Kind_High_Type_Attribute
   --Iir_Kind_Low_Type_Attribute
   --Iir_Kind_Ascending_Type_Attribute
   --Iir_Kind_Image_Attribute
   --Iir_Kind_Value_Attribute
   --Iir_Kind_Pos_Attribute
   --Iir_Kind_Val_Attribute
   --Iir_Kind_Succ_Attribute
   --Iir_Kind_Pred_Attribute
   --Iir_Kind_Leftof_Attribute
   --Iir_Kind_Rightof_Attribute
   --Iir_Kind_Left_Array_Attribute
   --Iir_Kind_Right_Array_Attribute
   --Iir_Kind_High_Array_Attribute
   --Iir_Kind_Low_Array_Attribute
   --Iir_Kind_Range_Array_Attribute
   --Iir_Kind_Reverse_Range_Array_Attribute
   --Iir_Kind_Length_Array_Attribute
   --Iir_Kind_Ascending_Array_Attribute
   --Iir_Kind_Delayed_Attribute
   --Iir_Kind_Stable_Attribute
   --Iir_Kind_Quiet_Attribute
   --Iir_Kind_Transaction_Attribute
   --Iir_Kind_Event_Attribute
   --Iir_Kind_Active_Attribute
   --Iir_Kind_Last_Event_Attribute
   --Iir_Kind_Last_Active_Attribute
   --Iir_Kind_Last_Value_Attribute
   --Iir_Kind_Driving_Attribute
   --Iir_Kind_Driving_Value_Attribute
   --Iir_Kind_Behavior_Attribute
   --Iir_Kind_Structure_Attribute
   --Iir_Kind_Simple_Name_Attribute
   --Iir_Kind_Instance_Name_Attribute
     Iir_Kind_Path_Name_Attribute;

   subtype Iir_Kinds_Attribute is Iir_Kind range
     Iir_Kind_Base_Attribute ..
     Iir_Kind_Path_Name_Attribute;

   subtype Iir_Kinds_Type_Attribute is Iir_Kind range
     Iir_Kind_Left_Type_Attribute ..
   --Iir_Kind_Right_Type_Attribute
   --Iir_Kind_High_Type_Attribute
   --Iir_Kind_Low_Type_Attribute
     Iir_Kind_Ascending_Type_Attribute;

   subtype Iir_Kinds_Scalar_Type_Attribute is Iir_Kind range
     Iir_Kind_Pos_Attribute ..
   --Iir_Kind_Val_Attribute
   --Iir_Kind_Succ_Attribute
   --Iir_Kind_Pred_Attribute
   --Iir_Kind_Leftof_Attribute
     Iir_Kind_Rightof_Attribute;

   subtype Iir_Kinds_Array_Attribute is Iir_Kind range
     Iir_Kind_Left_Array_Attribute ..
   --Iir_Kind_Right_Array_Attribute
   --Iir_Kind_High_Array_Attribute
   --Iir_Kind_Low_Array_Attribute
   --Iir_Kind_Range_Array_Attribute
   --Iir_Kind_Reverse_Range_Array_Attribute
   --Iir_Kind_Length_Array_Attribute
     Iir_Kind_Ascending_Array_Attribute;

   subtype Iir_Kinds_Signal_Attribute is Iir_Kind range
     Iir_Kind_Delayed_Attribute ..
   --Iir_Kind_Stable_Attribute
   --Iir_Kind_Quiet_Attribute
     Iir_Kind_Transaction_Attribute;

   subtype Iir_Kinds_Signal_Value_Attribute is Iir_Kind range
     Iir_Kind_Event_Attribute ..
   --Iir_Kind_Active_Attribute
   --Iir_Kind_Last_Event_Attribute
   --Iir_Kind_Last_Active_Attribute
   --Iir_Kind_Last_Value_Attribute
   --Iir_Kind_Driving_Attribute
     Iir_Kind_Driving_Value_Attribute;

   subtype Iir_Kinds_Concurrent_Statement is Iir_Kind range
     Iir_Kind_Sensitized_Process_Statement ..
   --Iir_Kind_Process_Statement
   --Iir_Kind_Concurrent_Conditional_Signal_Assignment
   --Iir_Kind_Concurrent_Selected_Signal_Assignment
   --Iir_Kind_Concurrent_Assertion_Statement
   --Iir_Kind_Concurrent_Procedure_Call_Statement
   --Iir_Kind_Block_Statement
   --Iir_Kind_Generate_Statement
     Iir_Kind_Component_Instantiation_Statement;

   subtype Iir_Kinds_Concurrent_Signal_Assignment is Iir_Kind range
     Iir_Kind_Concurrent_Conditional_Signal_Assignment ..
     Iir_Kind_Concurrent_Selected_Signal_Assignment;

   subtype Iir_Kinds_Sequential_Statement is Iir_Kind range
     Iir_Kind_Signal_Assignment_Statement ..
   --Iir_Kind_Null_Statement
   --Iir_Kind_Assertion_Statement
   --Iir_Kind_Report_Statement
   --Iir_Kind_Wait_Statement
   --Iir_Kind_Variable_Assignment_Statement
   --Iir_Kind_Return_Statement
   --Iir_Kind_For_Loop_Statement
   --Iir_Kind_While_Loop_Statement
   --Iir_Kind_Next_Statement
   --Iir_Kind_Exit_Statement
   --Iir_Kind_Case_Statement
   --Iir_Kind_Procedure_Call
     Iir_Kind_If_Statement;

   subtype Iir_Kinds_Allocator is Iir_Kind range
     Iir_Kind_Allocator_By_Expression ..
     Iir_Kind_Allocator_By_Subtype;

   subtype Iir_Kinds_Clause is Iir_Kind range
     Iir_Kind_Library_Clause ..
     Iir_Kind_Use_Clause;

   subtype Iir_Kinds_Specification is Iir_Kind range
     Iir_Kind_Attribute_Specification ..
   --Iir_Kind_Disconnection_Specification
     Iir_Kind_Configuration_Specification;

   subtype Iir_Kinds_Declaration is Iir_Kind range
     Iir_Kind_Type_Declaration ..
   --Iir_Kind_Anonymous_Type_Declaration
   --Iir_Kind_Subtype_Declaration
   --Iir_Kind_Configuration_Declaration
   --Iir_Kind_Entity_Declaration
   --Iir_Kind_Package_Declaration
   --Iir_Kind_Package_Body
   --Iir_Kind_Architecture_Declaration
   --Iir_Kind_Unit_Declaration
   --Iir_Kind_Library_Declaration
   --Iir_Kind_Component_Declaration
   --Iir_Kind_Attribute_Declaration
   --Iir_Kind_Group_Template_Declaration
   --Iir_Kind_Group_Declaration
   --Iir_Kind_Element_Declaration
   --Iir_Kind_Non_Object_Alias_Declaration
   --Iir_Kind_Implicit_Function_Declaration
   --Iir_Kind_Function_Specification
   --Iir_Kind_Function_Spec_Body
   --Iir_Kind_Function_Body
   --Iir_Kind_Implicit_Procedure_Declaration
   --Iir_Kind_Procedure_Specification
   --Iir_Kind_Procedure_Spec_Body
   --Iir_Kind_Procedure_Body
   --Iir_Kind_Enumeration_Literal
   --Iir_Kind_File_Declaration
   --Iir_Kind_Object_Alias_Declaration
   --Iir_Kind_Guard_Signal_Declaration
   --Iir_Kind_Signal_Declaration
   --Iir_Kind_Variable_Declaration
   --Iir_Kind_Constant_Declaration
   --Iir_Kind_Iterator_Declaration
   --Iir_Kind_Constant_Interface_Declaration
   --Iir_Kind_Variable_Interface_Declaration
   --Iir_Kind_Signal_Interface_Declaration
     Iir_Kind_File_Interface_Declaration;

   -------------------------------------
   -- Types and subtypes declarations --
   -------------------------------------

   -- Level 1 base class.
   type Iir_Node (Kind: Iir_Kind := Iir_Kind_Error) is private;
   type Iir is access Iir_Node;
   subtype Iir_List is Iir;
   Null_Iir : constant Iir;
   Null_Iir_List : constant Iir_List;
   Error_Iir : constant Iir;

   -- This is used only for lists.
   type Iir_Array is array (Natural range <>) of Iir;
   type Iir_Array_Acc is access Iir_Array;
   procedure Free is new Ada.Unchecked_Deallocation
     (Object => Iir_Array, Name => Iir_Array_Acc);

   --  A date is used for analysis order.  All design units from a library
   --  are ordered according to the date.
   type Date_Type is new Natural;
   --  The unit is obseleted (ie replaced) by a more recently analyzed design
   --  unit.another design unit.
   --  If another design unit depends (directly or not) on an obseleted design
   --  unit, it is also obselete, and cannot be defined.
   Date_Obsolete      : constant Date_Type := 0;
   --  The unit was not analyzed.
   Date_Not_Analyzed  : constant Date_Type := 1;
   --  The unit has been analyzed but it has bad dependences.
   Date_Bad_Analyze   : constant Date_Type := 2;
   --  The unit has been parsed but not analyzed.
   Date_Parsed        : constant Date_Type := 4;
   --  This unit has just been analyzed and should be marked at the last
   --  analyzed unit.
   Date_Analyzed      : constant Date_Type := 5;
   subtype Date_Valid is Date_Type range 10 .. Date_Type'Last;

   -- design file
   subtype Iir_Design_File is Iir (Iir_Kind_Design_File);

   subtype Iir_Design_Unit is Iir (Iir_Kind_Design_Unit);

   subtype Iir_Library_Clause is Iir (Iir_Kind_Library_Clause);

   -- Literals.
   subtype Iir_Text_Literal is Iir;

   subtype Iir_Character_Literal is Iir (Iir_Kind_Character_Literal);

   subtype Iir_Integer_Literal is Iir (Iir_Kind_Integer_Literal);

   subtype Iir_Floating_Point_Literal is
     Iir (Iir_Kind_Floating_Point_Literal);

   subtype Iir_String_Literal is Iir (Iir_Kind_String_Literal);

   subtype Iir_Bit_String_Literal is Iir (Iir_Kind_Bit_String_Literal);

   subtype Iir_Null_Literal is Iir (Iir_Kind_Null_Literal);

   subtype Iir_Physical_Int_Literal is Iir (Iir_Kind_Physical_Int_Literal);

   subtype Iir_Physical_Fp_Literal is Iir (Iir_Kind_Physical_Fp_Literal);

   subtype Iir_Enumeration_Literal is Iir (Iir_Kind_Enumeration_Literal);

   subtype Iir_Simple_Aggregate is Iir (Iir_Kind_Simple_Aggregate);

   subtype Iir_Enumeration_Type_Definition is
     Iir (Iir_Kind_Enumeration_Type_Definition);

   subtype Iir_Enumeration_Subtype_Definition is
     Iir (Iir_Kind_Enumeration_Subtype_Definition);

   subtype Iir_Range_Expression is Iir (Iir_Kind_Range_Expression);

   subtype Iir_Integer_Subtype_Definition is
     Iir (Iir_Kind_Integer_Subtype_Definition);

   subtype Iir_Integer_Type_Definition is
     Iir (Iir_Kind_Integer_Type_Definition);

   subtype Iir_Floating_Subtype_Definition is
     Iir (Iir_Kind_Floating_Subtype_Definition);

   subtype Iir_Floating_Type_Definition is
     Iir (Iir_Kind_Floating_Type_Definition);

   subtype Iir_Array_Type_Definition is Iir (Iir_Kind_Array_Type_Definition);

   subtype Iir_Record_Type_Definition is Iir (Iir_Kind_Record_Type_Definition);

   subtype Iir_Subtype_Definition is Iir (Iir_Kind_Subtype_Definition);

   subtype Iir_Array_Subtype_Definition is
     Iir (Iir_Kind_Array_Subtype_Definition);

   subtype Iir_Unconstrained_Array_Subtype_Definition is
     Iir (Iir_Kind_Unconstrained_Array_Subtype_Definition);

   subtype Iir_Physical_Type_Definition is
     Iir (Iir_Kind_Physical_Type_Definition);

   subtype Iir_Physical_Subtype_Definition is
     Iir (Iir_Kind_Physical_Subtype_Definition);

   subtype Iir_Access_Type_Definition is
     Iir (Iir_Kind_Access_Type_Definition);

   subtype Iir_Access_Subtype_Definition is
     Iir (Iir_Kind_Access_Subtype_Definition);

   subtype Iir_File_Type_Definition is
     Iir (Iir_Kind_File_Type_Definition);

   -- Tuples.
   subtype Iir_Proxy is Iir (Iir_Kind_Proxy);

   subtype Iir_Waveform_Element is Iir (Iir_Kind_Waveform_Element);

   subtype Iir_Conditional_Waveform is Iir (Iir_Kind_Conditional_Waveform);

   subtype Iir_Association_Element_By_Expression is
     Iir (Iir_Kind_Association_Element_By_Expression);

   subtype Iir_Association_Element_By_Individual is
     Iir (Iir_Kind_Association_Element_By_Individual);

   subtype Iir_Association_Element_Open is
     Iir (Iir_Kind_Association_Element_Open);

   subtype Iir_Signature is Iir (Iir_Kind_Signature);

   subtype Iir_Unit_Declaration is Iir (Iir_Kind_Unit_Declaration);

   subtype Iir_Entity_Aspect_Entity is Iir (Iir_Kind_Entity_Aspect_Entity);

   subtype Iir_Entity_Aspect_Configuration is
     Iir (Iir_Kind_Entity_Aspect_Configuration);

   subtype Iir_Entity_Aspect_Open is Iir (Iir_Kind_Entity_Aspect_Open);

   subtype Iir_Block_Configuration is Iir (Iir_Kind_Block_Configuration);

   subtype Iir_Block_Header is Iir (Iir_Kind_Block_Header);

   subtype Iir_Component_Configuration is
     Iir (Iir_Kind_Component_Configuration);

   subtype Iir_Binding_Indication is Iir (Iir_Kind_Binding_Indication);

   subtype Iir_Entity_Class is Iir (Iir_Kind_Entity_Class);

   subtype Iir_Attribute_Specification is
     Iir (Iir_Kind_Attribute_Specification);

   subtype Iir_Attribute_Value is Iir (Iir_Kind_Attribute_Value);

   subtype Iir_Selected_Element is Iir (Iir_Kind_Selected_Element);

   subtype Iir_Implicit_Dereference is Iir (Iir_Kind_Implicit_Dereference);

   subtype Iir_Aggregate_Info is Iir (Iir_Kind_Aggregate_Info);

   subtype Iir_Disconnection_Specification is
     Iir (Iir_Kind_Disconnection_Specification);

   -- Lists.

   subtype Iir_Index_List is Iir (Iir_Kind_Index_List);

   subtype Iir_Design_Unit_List is Iir (Iir_Kind_Design_Unit_List);

   subtype Iir_Enumeration_Literal_List is
     Iir (Iir_Kind_Enumeration_Literal_List);

   subtype Iir_Designator_List is Iir (Iir_Kind_Designator_List);

   subtype Iir_Driver_List is Iir (Iir_Kind_Driver_List);

   subtype Iir_Attribute_Value_List is Iir (Iir_Kind_Attribute_Value_List);

   subtype Iir_Overload_List is Iir (Iir_Kind_Overload_List);

   subtype Iir_Group_Constituent_List is Iir (Iir_Kind_Group_Constituent_List);

   subtype Iir_Callees_List is Iir (Iir_Kind_Callees_List);

   -- Declaration and children.
   subtype Iir_Entity_Declaration is Iir (Iir_Kind_Entity_Declaration);

   subtype Iir_Signal_Interface_Declaration is
     Iir (Iir_Kind_Signal_Interface_Declaration);

   subtype Iir_Architecture_Declaration is
     Iir (Iir_Kind_Architecture_Declaration);

   subtype Iir_Configuration_Declaration is
     Iir (Iir_Kind_Configuration_Declaration);

   subtype Iir_Type_Declaration is Iir (Iir_Kind_Type_Declaration);

   subtype Iir_Anonymous_Type_Declaration is
     Iir (Iir_Kind_Anonymous_Type_Declaration);

   subtype Iir_Subtype_Declaration is Iir (Iir_Kind_Subtype_Declaration);

   subtype Iir_Package_Declaration is Iir (Iir_Kind_Package_Declaration);
   subtype Iir_Package_Body is Iir (Iir_Kind_Package_Body);

   subtype Iir_Library_Declaration is Iir (Iir_Kind_Library_Declaration);

   subtype Iir_Function_Declaration is Iir (Iir_Kind_Function_Declaration);

   subtype Iir_Function_Body is Iir (Iir_Kind_Function_Body);

   subtype Iir_Procedure_Declaration is Iir (Iir_Kind_Procedure_Declaration);

   subtype Iir_Procedure_Body is Iir (Iir_Kind_Procedure_Body);

   subtype Iir_Implicit_Function_Declaration is
     Iir (Iir_Kind_Implicit_Function_Declaration);
   subtype Iir_Implicit_Procedure_Declaration is
     Iir (Iir_Kind_Implicit_Procedure_Declaration);

   subtype Iir_Use_Clause is Iir (Iir_Kind_Use_Clause);

   subtype Iir_Constant_Declaration is Iir (Iir_Kind_Constant_Declaration);

   subtype Iir_Iterator_Declaration is Iir (Iir_Kind_Iterator_Declaration);

   subtype Iir_Constant_Interface_Declaration is
     Iir (Iir_Kind_Constant_Interface_Declaration);

   subtype Iir_Variable_Interface_Declaration is
     Iir (Iir_Kind_Variable_Interface_Declaration);

   subtype Iir_File_Interface_Declaration is
     Iir (Iir_Kind_File_Interface_Declaration);

   subtype Iir_Guard_Signal_Declaration is
     Iir (Iir_Kind_Guard_Signal_Declaration);

   subtype Iir_Signal_Declaration is Iir (Iir_Kind_Signal_Declaration);

   subtype Iir_Variable_Declaration is Iir (Iir_Kind_Variable_Declaration);

   subtype Iir_Component_Declaration is Iir (Iir_Kind_Component_Declaration);

   subtype Iir_Element_Declaration is Iir (Iir_Kind_Element_Declaration);

   subtype Iir_Object_Alias_Declaration is
     Iir (Iir_Kind_Object_Alias_Declaration);

   subtype Iir_Non_Object_Alias_Declaration is
     Iir (Iir_Kind_Non_Object_Alias_Declaration);

   subtype Iir_Interface_Declaration is Iir;

   subtype Iir_Configuration_Specification is
     Iir (Iir_Kind_Configuration_Specification);

   subtype Iir_File_Declaration is Iir (Iir_Kind_File_Declaration);

   subtype Iir_Attribute_Declaration is Iir (Iir_Kind_Attribute_Declaration);

   subtype Iir_Group_Template_Declaration is
     Iir (Iir_Kind_Group_Template_Declaration);

   subtype Iir_Group_Declaration is Iir (Iir_Kind_Group_Declaration);

   -- concurrent_statement and children.
   subtype Iir_Concurrent_Statement is Iir;

   subtype Iir_Concurrent_Conditional_Signal_Assignment is
     Iir (Iir_Kind_Concurrent_Conditional_Signal_Assignment);

   subtype Iir_Sensitized_Process_Statement is
     Iir (Iir_Kind_Sensitized_Process_Statement);

   subtype Iir_Process_Statement is Iir (Iir_Kind_Process_Statement);

   subtype Iir_Component_Instantiation_Statement is
     Iir (Iir_Kind_Component_Instantiation_Statement);

   subtype Iir_Block_Statement is Iir (Iir_Kind_Block_Statement);

   subtype Iir_Generate_Statement is Iir (Iir_Kind_Generate_Statement);

   -- sequential statements.
   subtype Iir_If_Statement is Iir (Iir_Kind_If_Statement);

   subtype Iir_Elsif is Iir (Iir_Kind_Elsif);

   subtype Iir_For_Loop_Statement is Iir (Iir_Kind_For_Loop_Statement);

   subtype Iir_While_Loop_Statement is Iir (Iir_Kind_While_Loop_Statement);

   subtype Iir_Exit_Statement is Iir (Iir_Kind_Exit_Statement);
   subtype Iir_Next_Statement is Iir (Iir_Kind_Next_Statement);

   subtype Iir_Variable_Assignment_Statement is
     Iir (Iir_Kind_Variable_Assignment_Statement);

   subtype Iir_Signal_Assignment_Statement is
     Iir (Iir_Kind_Signal_Assignment_Statement);

   subtype Iir_Assertion_Statement is Iir (Iir_Kind_Assertion_Statement);

   subtype Iir_Report_Statement is Iir (Iir_Kind_Report_Statement);

   subtype Iir_Wait_Statement is Iir (Iir_Kind_Wait_Statement);

   subtype Iir_Return_Statement is Iir (Iir_Kind_Return_Statement);

   subtype Iir_Case_Statement is Iir (Iir_Kind_Case_Statement);

   subtype Iir_Procedure_Call is Iir (Iir_Kind_Procedure_Call);

   -- expression and children.
   subtype Iir_Expression is Iir;

   subtype Iir_Function_Call is Iir (Iir_Kind_Function_Call);

   subtype Iir_Aggregate is Iir (Iir_Kind_Aggregate);

   subtype Iir_Qualified_Expression is Iir (Iir_Kind_Qualified_Expression);

   subtype Iir_Type_Conversion is Iir (Iir_Kind_Type_Conversion);

   subtype Iir_Allocator_By_Expression is
     Iir (Iir_Kind_Allocator_By_Expression);

   subtype Iir_Allocator_By_Subtype is Iir (Iir_Kind_Allocator_By_Subtype);

   -- names.
   subtype Iir_Simple_Name is Iir (Iir_Kind_Simple_Name);

   subtype Iir_Slice_Name is Iir (Iir_Kind_Slice_Name);

   subtype Iir_Selected_Name is Iir (Iir_Kind_Selected_Name);

   subtype Iir_Selected_By_All_Name is Iir (Iir_Kind_Selected_By_All_Name);

   subtype Iir_Indexed_Name is Iir (Iir_Kind_Indexed_Name);

   subtype Iir_Parenthesis_Name is Iir (Iir_Kind_Parenthesis_Name);

   -- attributes.
   subtype Iir_Attribute_Name is Iir (Iir_Kind_Attribute_Name);

   -- This kind can't be handled.
   --procedure Error_Kind (Msg: String; Kind: Iir_Kind);
   procedure Error_Kind (Msg: String; An_Iir: in Iir);
   procedure Error_Kind (Msg: String; Def : Iir_Predefined_Functions);
   pragma No_Return (Error_Kind);

   -- General methods.

   -- Get the kind of the iir.
   function Get_Kind (An_Iir: Iir) return Iir_Kind;

   --  Create a new IIR of kind NEW_KIND, and copy fields from SRC to this
   --  iir.  Src fields are cleaned.
   function Clone_Iir (Src: Iir; New_Kind : Iir_Kind) return Iir;

   -- About the location, ie the position of the iir in the source file.
   -- The location refers to the first token of the iir, with the following
   -- exception:
   -- For an expression, it refers to the position of the operator.
   function Get_Column_Number (Target: Iir) return Natural;
   function Get_Line_Number (Target: Iir) return Natural;
   function Get_File_Name (Target: Iir) return Name_Id;

   procedure Set_Location (Target: Iir; Location: Location_Type);
   function Get_Location (Target: Iir) return Location_Type;

   procedure Location_Copy (Target: Iir; Src: Iir);

   function  Get_Back_End_Info (Target: Iir) return System.Address;
   procedure Set_Back_End_Info (Target: Iir; Addr: System.Address);

   function Create_Iir (Kind: Iir_Kind) return Iir;
   function Create_Iir_Error return Iir;
   procedure Free_Iir (Target: in out Iir);

   --  Design units contained in a design file.
   --  Field: Field1
   function Get_Design_Unit_Chain (Design: Iir_Design_File)
                                  return Iir_Design_Unit;
   procedure Set_Design_Unit_Chain
     (Design : Iir_Design_File; Chain : Iir_Design_Unit);

   --  Library declaration of a library clause.
   --  Field: Field2
   function Get_Library_Declaration (Design : Iir)
     return Iir_Library_Declaration;
   procedure Set_Library_Declaration
     (Design : Iir; Library : Iir_Library_Declaration);

   -- File time stamp is the system time of the file last modification.
   --  Field: Field2
   function Get_File_Time_Stamp (Design: Iir_Design_File)
     return Time_Stamp_Acc;
   procedure Set_File_Time_Stamp
     (Design: Iir_Design_File; Stamp: Time_Stamp_Acc);

   -- Time stamp of the last analysis system time.
   --  Field: Field3
   function Get_Analysis_Time_Stamp (Design: Iir_Design_File)
     return Time_Stamp_Acc;
   procedure Set_Analysis_Time_Stamp
     (Design: Iir_Design_File; Stamp: Time_Stamp_Acc);

   --  Identifier for the design file file name.
   --  Field: Nbr3
   function Get_Design_File_Filename (File : Iir_Design_File) return Name_Id;
   procedure Set_Design_File_Filename (File : Iir_Design_File; Name : Name_Id);

   --  Directory of a design file.
   --  Field: Nbr2
   function Get_Design_File_Directory (File : Iir_Design_File) return Name_Id;
   procedure Set_Design_File_Directory (File : Iir_Design_File; Dir : Name_Id);

   --  The parent of a design unit is a design file.
   --  Field: Parent
   function Get_Design_File (Unit : Iir_Design_Unit) return Iir_Design_File;
   procedure Set_Design_File (Unit : Iir_Design_Unit; File : Iir_Design_File);

   --  Design files of a library.
   --  Field: Field1
   function Get_Design_File_Chain (Library: Iir_Library_Declaration)
     return Iir_Design_File;
   procedure Set_Design_File_Chain (Library: Iir_Library_Declaration;
                                    Chain : Iir_Design_File);

   --  System directory where the library is stored.
   --  Field: Nbr2
   function Get_Library_Directory (Library : Iir_Library_Declaration)
     return Name_Id;
   procedure Set_Library_Directory (Library : Iir_Library_Declaration;
                                    Dir : Name_Id);

   -- Symbolic date, used to order design units in a library.
   --  Field: Field5
   function Get_Date (Target: Iir) return Date_Type;
   procedure Set_Date (Target: Iir; Date: Date_Type);

   --  Chain of context clauses.
   --  Field: Field1
   function Get_Context_Items (Design_Unit: Iir_Design_Unit)
     return Iir;
   procedure Set_Context_Items (Design_Unit: Iir_Design_Unit;
                                Items_Chain : Iir);

   --  List of design units on which the design unit depends.
   --  Field: Field3
   function Get_Dependence_List (Design_Unit: Iir_Design_Unit)
     return Iir_Design_Unit_List;
   procedure Set_Dependence_List
     (Design_Unit: Iir_Design_Unit; List : Iir_Design_Unit_List);

   --  If TRUE, design_unit is in memory, else it is stubbed.
   --  Field: Flag1
   function Get_Loaded_Flag (Design_Unit : Iir_Design_Unit) return Boolean;
   procedure Set_Loaded_Flag (Design_Unit : Iir_Design_Unit; Flag : Boolean);

   --  If TRUE, the target of the signal assignment is guarded.
   --  If FALSE, the target is not guarded.
   --  This is determined during sem by examining the declaration(s) of the
   --  target (there may be severals declarations in the case of a aggregate
   --  target).
   --  If UNKNOWN, this is not determined at compile time but at run-time.
   --  This is the case for formal signal interfaces of subprograms.
   --  Field: State1
   function Get_Guarded_Target_State (Stmt : Iir) return Tri_State_Type;
   procedure Set_Guarded_Target_State (Stmt : Iir; State : Tri_State_Type);

   --  Library unit of a design unit.
   --  Field: Field2
   function Get_Library_Unit (Design_Unit: Iir_Design_Unit) return Iir;
   procedure Set_Library_Unit (Design_Unit: Iir_Design_Unit;
                               Library_Unit: Iir);

   -- Set the line and the offset in the line, only for the library manager.
   -- This is valid until the file is really loaded in memory.  On loading,
   -- location will contain all this informations.
   --  Field: Field1
   --  Field: Field6
   --  Field: Field7
   procedure Set_Pos_Line_Off (Design_Unit: Iir_Design_Unit;
                               Pos : Source_Ptr; Line, Off: Natural);
   procedure Get_Pos_Line_Off (Design_Unit: Iir_Design_Unit;
                               Pos : out Source_Ptr; Line, Off: out Natural);


   --  literals.

   --  Value of an integer/physical literal.
   --  Field: Nbr3
   function Get_Value (Lit: Iir) return Iir_Int64;
   procedure Set_Value (Lit: Iir; Val: Iir_Int64);

   --  Value of a floating point literal.
   --  Field: Nbr3
   function Get_Fp_Value (Lit: Iir) return Iir_Fp64;
   procedure Set_Fp_Value (Lit: Iir; Val: Iir_Fp64);

   --  Declaration of the literal.
   --  This is used to retrieve the genuine enumeration literal for literals
   --  created from static expression.
   --  Field: Field2
   function Get_Enumeration_Decl (Target : Iir_Enumeration_Literal)
                                 return Iir_Enumeration_Literal;
   procedure Set_Enumeration_Decl (Target : Iir_Enumeration_Literal;
                                   Lit : Iir_Enumeration_Literal);

   --  List of elements of a simple aggregate.
   --  Field: Field2
   function Get_Simple_Aggregate_List (Target : Iir_Simple_Aggregate)
                                      return Iir_List;
   procedure Set_Simple_Aggregate_List (Target: Iir_Simple_Aggregate;
                                        List : Iir_List);

   -- The logarithm of the base (1, 3 or 4) of a bit string.
   --  Field: Nbr2
   function Get_Bit_String_Base (Lit : Iir_Bit_String_Literal)
                                 return Natural;
   procedure Set_Bit_String_Base (Lit : Iir_Bit_String_Literal;
                                  Base : Natural);

   --  The enumeration literal which defines the '0' and '1' value.
   --  Field: Field4
   function Get_Bit_String_0 (Lit : Iir_Bit_String_Literal)
     return Iir_Enumeration_Literal;
   procedure Set_Bit_String_0 (Lit : Iir_Bit_String_Literal;
                               El : Iir_Enumeration_Literal);

   --  Field: Field5
   function Get_Bit_String_1 (Lit : Iir_Bit_String_Literal)
     return Iir_Enumeration_Literal;
   procedure Set_Bit_String_1 (Lit : Iir_Bit_String_Literal;
                               El : Iir_Enumeration_Literal);

   --  The origin of a literal can be null_iir for a literal generated by the
   --  parser, or a node which was statically evaluated to this literal.
   --  Such nodes are created by eval_expr.
   --  Field: Field5
   function Get_Literal_Origin (Lit: Iir) return Iir;
   procedure Set_Literal_Origin (Lit: Iir; Orig : Iir);

   --  tuples.

   function Create_Proxy (Proxy: Iir) return Iir_Proxy;

   --  Field: Field1
   function Get_Proxy (Proxy: Iir_Proxy) return Iir;
   procedure Set_Proxy (Target: Iir_Proxy; Proxy: Iir);

   --  Field: Nbr3
   procedure Set_Entity_Kind (Target : Iir; Kind : Tokens.Token_Type);
   function Get_Entity_Kind (Target : Iir) return Tokens.Token_Type;

   --  Field: Field1
   function Get_Entity_Name_List (Target : Iir_Attribute_Specification)
     return Iir;
   procedure Set_Entity_Name_List (Target : Iir_Attribute_Specification;
                                   Names : Iir);

   --  Field: Field2
   function Get_Attribute_Designator (Target : Iir_Attribute_Specification)
     return Iir;
   procedure Set_Attribute_Designator
     (Target: Iir_Attribute_Specification; Designator : Iir);

   --  Field: Field2
   function Get_Attribute_Specification
     (Val : Iir_Attribute_Value) return Iir_Attribute_Specification;
   procedure Set_Attribute_Specification
     (Val : Iir_Attribute_Value;
      Attr : Iir_Attribute_Specification);

   --  Field: Field2
   function Get_Signal_List (Target : Iir_Disconnection_Specification)
     return Iir;
   procedure Set_Signal_List (Target : Iir_Disconnection_Specification;
                              List : Iir);

   --  Field: Field3
   function Get_Designated_Entity (Val : Iir_Attribute_Value) return Iir;
   procedure Set_Designated_Entity (Val : Iir_Attribute_Value; Entity : Iir);

   --  Field: Field1
   procedure Set_Formal (Target: Iir; Formal: Iir);
   function Get_Formal (Target: Iir) return Iir;

   --  Field: Field2
   procedure Set_Actual (Target: Iir; Actual: Iir);
   function Get_Actual (Target: Iir) return Iir;

   --  Field: Field3
   procedure Set_Associated_Formal (Target: Iir; Formal : Iir);
   function Get_Associated_Formal (Target: Iir) return Iir;

   --  Field: Field4
   procedure Set_In_Conversion (Target : Iir; Conv : Iir);
   function Get_In_Conversion (Target : Iir) return Iir;

   --  Field: Field5
   procedure Set_Out_Conversion (Target : Iir; Conv : Iir);
   function Get_Out_Conversion (Target : Iir) return Iir;

   --  This flag is set when the formal is associated in whole (ie, not
   --  individually).
   --  Field: Flag1
   procedure Set_Whole_Association_Flag (Target : Iir; Flag : Boolean);
   function Get_Whole_Association_Flag (Target : Iir) return Boolean;

   --  Field: Field1
   function Get_We_Value (We: Iir_Waveform_Element) return Iir;
   procedure Set_We_Value (We: Iir_Waveform_Element; An_Iir: Iir);

   --  Field: Field2
   function Get_Time (We: Iir_Waveform_Element) return Iir;
   procedure Set_Time (We: Iir_Waveform_Element; An_Iir: Iir);

   --  Field: Field1
   procedure Set_Associated (Target: Iir; Associated: Iir);
   function Get_Associated (Target: Iir) return Iir;

   --  Field: Field2
   procedure Set_Architecture (Target: Iir_Entity_Aspect_Entity; Arch : Iir);
   function Get_Architecture (Target: Iir_Entity_Aspect_Entity) return Iir;

   --  Field: Field1
   procedure Set_Block_Specification (Target: Iir; Block: Iir);
   function Get_Block_Specification (Target: Iir) return Iir;

   --  Return the link of the previous block_configuration of a
   --  block_configuration.
   --  This single linked list is used to list all the block_configuration that
   --  configuration the same block (which can only be an iterative generate
   --  statement).
   --  All elements of this list must belong to the same block configuration.
   --  The order is not important.
   --  Field: Field4
   procedure Set_Prev_Block_Configuration
     (Target : Iir_Block_Configuration; Block : Iir_Block_Configuration);
   function Get_Prev_Block_Configuration (Target : Iir_Block_Configuration)
     return Iir_Block_Configuration;

   --  Field: Field2
   function Get_Configuration_Item_Chain (Target: Iir) return Iir;
   procedure Set_Configuration_Item_Chain (Target : Iir; Chain : Iir);

   --  Field: Attr
   function Get_Attribute_Value_List (Target : Iir)
                                      return Iir_Attribute_Value_List;
   procedure Set_Attribute_Value_List (Target : Iir;
                                       List : Iir_Attribute_Value_List);

   -- lists.

   procedure Append_Element (List: Iir; Element: Iir);

   -- Get the N th element in list, starting from 0.
   -- Return the element or null_iir, if beyond bounds.
   function Get_Nth_Element (List: Iir_List; N: Natural) return Iir;

   function Get_Last_Element (List: Iir) return Iir;

   function Get_First_Element (List: Iir) return Iir;

   procedure Replace_Nth_Element (List: Iir; N: Natural; El: Iir);

   procedure Add_Element (List: Iir; El: Iir);

   -- Return the number of elements in the list.
   -- This is also 1 + the position of the last element.
   function Get_Nbr_Elements (List: Iir_List) return Natural;
   pragma Inline (Get_Nbr_Elements);

   --  Same as get_nbr_elements but returns 0 if LIST is NULL_IIR.
   function Get_Nbr_Elements_Safe (List: in Iir_List) return Natural;

   -- Set the number of elements in the list.
   -- Can be used only to shrink the list.
   procedure Set_Nbr_Elements (List: Iir_List; N: Natural);

   procedure Remove_Nth_Element (List: Iir; N: Natural);

   function Get_Last_Element_Position (List: Iir) return Integer;

   procedure Empty_List (List: Iir);

   --  Field: Field1
   procedure Set_Entity (Target : Iir; Entity: Iir);
   function Get_Entity (Decl: Iir) return Iir;

   --  The package declaration corresponding to the body.
   --  Field: Field1
   procedure Set_Package (Package_Body: Iir; Decl: Iir_Package_Declaration);
   function Get_Package (Package_Body: Iir) return Iir_Package_Declaration;

   --  If true, the package need a body.
   --  Field: Flag1
   procedure Set_Need_Body (Decl : Iir_Package_Declaration; Flag : Boolean);
   function Get_Need_Body (Decl : Iir_Package_Declaration) return Boolean;

   --  Field: Field2
   procedure Set_Block_Configuration
     (Target: Iir; Block: Iir_Block_Configuration);
   function Get_Block_Configuration (Target: Iir)
     return Iir_Block_Configuration;

   --  Field: Field2
   procedure Set_Concurrent_Statement_Chain (Target: Iir; First : Iir);
   function Get_Concurrent_Statement_Chain (Target: Iir) return Iir;

   --  Field: Chain
   procedure Set_Chain (Target : Iir; Value : Iir);
   function Get_Chain (Target : Iir) return Iir;

   --  Field: Field1
   procedure Set_Port_Chain (Target: Iir; Chain: Iir);
   function Get_Port_Chain (Target: Iir) return Iir;

   --  Field: Field4
   procedure Set_Generic_Chain (Target: Iir; Generics: Iir);
   function Get_Generic_Chain (Target: Iir) return Iir;

   --  Field: Field1
   procedure Set_Type (Target: Iir; Atype: Iir);
   function Get_Type (Target: Iir) return Iir;

   --  The subtype definition associated with the type declaration (if any).
   --  Field: Field2
   procedure Set_Subtype_Definition (Target : Iir_Anonymous_Type_Declaration;
                                     Def : Iir);
   function Get_Subtype_Definition (Target : Iir_Anonymous_Type_Declaration)
                                   return Iir;

   --  Field: Odigit_2
   procedure Set_Mode (Target: Iir; Mode: Iir_Mode);
   function Get_Mode (Target: Iir) return Iir_Mode;

   --  Field: State1
   procedure Set_Signal_Kind (Target: Iir; Signal_Kind: Iir_Signal_Kind);
   function Get_Signal_Kind (Target: Iir) return Iir_Signal_Kind;

   --  The base name of a name is the node at the origin of the name.
   --  The base name is a declaration (signal, object, constant or interface),
   --  a selected_by_all name, an implicit_dereference name.
   --  Field: Field4
   procedure Set_Base_Name (Target : Iir; Name : Iir);
   function Get_Base_Name (Target : Iir) return Iir;

   --  Field: Field4
   function Get_Interface_Declaration_Chain (Target: Iir) return Iir;
   procedure Set_Interface_Declaration_Chain (Target : Iir; Chain : Iir);

   --  Field: Field4
   function Get_Subprogram_Specification (Target: Iir) return Iir;
   procedure Set_Subprogram_Specification (Target: Iir; Spec : Iir);

   --  Field: Field2
   function Get_Sequential_Statement_Chain (Target: Iir) return Iir;
   procedure Set_Sequential_Statement_Chain (Target : Iir; Chain : Iir);

   --  Field: Field2
   function Get_Subprogram_Body (Target: Iir) return Iir;
   procedure Set_Subprogram_Body (Target: Iir; A_Body: Iir);

   --  Field: Field1
   procedure Set_Return_Type (Target: Iir; Decl: Iir);
   function Get_Return_Type (Target: Iir) return Iir;

   --  Code of an implicit subprogram definition.
   --  Field: Field7
   procedure Set_Implicit_Definition (Target : Iir;
                                      Def : Iir_Predefined_Functions);
   function Get_Implicit_Definition (Target : Iir)
     return Iir_Predefined_Functions;

   --  For an implicit subprogram, the type_reference is the type declaration
   --  for which the implicit subprogram was defined.
   --  Field: Field2
   procedure Set_Type_Reference (Target : Iir; Decl : Iir);
   function Get_Type_Reference (Target : Iir) return Iir;

   --  Get the default value of an object declaration.
   --  Null_iir if no default value.
   --  Field: Field2
   procedure Set_Default_Value (Target: Iir; Value: Iir);
   function Get_Default_Value (Target: Iir) return Iir;

   --  The deferred_declaration field points to the deferred constant
   --  declaration for a full constant declaration, or is null_iir for a
   --  usual or deferred constant declaration.
   --  Set only during sem.
   --  Field: Field3
   procedure Set_Deferred_Declaration
     (Target : Iir_Constant_Declaration; Decl : Iir);
   function Get_Deferred_Declaration (Target : Iir_Constant_Declaration)
     return Iir;

   --  The deferred_declaration_flag must be set if the constant declaration is
   --  a deferred_constant declaration.
   --  Set only during sem.
   --  Field: Flag1
   procedure Set_Deferred_Declaration_Flag
     (Target : Iir_Constant_Declaration; Flag: Boolean);
   function Get_Deferred_Declaration_Flag (Target : Iir_Constant_Declaration)
     return Boolean;

   --  If true, the variable is declared shared.
   --  Field: Flag2
   procedure Set_Shared_Flag (Target : Iir_Variable_Declaration;
                              Shared : Boolean);
   function Get_Shared_Flag (Target : Iir_Variable_Declaration)
     return Boolean;

   --  Get the design unit in which the target is declared.
   --  For a library unit, this is to get the design unit node.
   --  Field: Parent
   function Get_Design_Unit (Target : Iir) return Iir_Design_Unit;
   procedure Set_Design_Unit (Target : Iir; Unit : Iir_Design_Unit);

   --  Field: Field3
   procedure Set_Block_Statement (Target: Iir; Block: Iir_Block_Statement);
   function Get_Block_Statement (Target: Iir) return Iir_Block_Statement;

   --  For a non-resolved signal: null_iir if the signal has no driver, or
   --  a process/concurrent_statement for which the signal should have a
   --  driver.  This is used to catch at analyse time unresolved signals with
   --  several drivers.
   --  Field: Field5
   procedure Set_Signal_Driver (Target : Iir_Signal_Declaration; Driver : Iir);
   function Get_Signal_Driver (Target : Iir_Signal_Declaration) return Iir;

   --  Field: Field3
   function Get_Declaration_Chain (Target: Iir) return Iir;
   procedure Set_Declaration_Chain (Target: Iir; Decls : Iir);

   --  Field: Field2
   procedure Set_File_Logical_Name (Target: Iir_File_Declaration; Name: Iir);
   function Get_File_Logical_Name (Target: Iir_File_Declaration)
                                   return Iir;

   --  Field: Field3
   procedure Set_File_Open_Kind (Target: Iir_File_Declaration; Kind: Iir);
   function Get_File_Open_Kind (Target: Iir_File_Declaration)
                                return Iir;

   --  Field: Field6
   procedure Set_Element_Position (Target: Iir_Element_Declaration;
                                   Pos: Iir_Index32);
   function Get_Element_Position (Target: Iir_Element_Declaration)
     return Iir_Index32;

   --  Field: Field2
   procedure Set_Selected_Element (Target: Iir_Selected_Element;
                                   El : Iir_Element_Declaration);
   function Get_Selected_Element (Target: Iir_Selected_Element)
                                  return Iir_Element_Declaration;

   --  Selected names of an use_clause are chained.
   --  Field: Field2
   procedure Set_Use_Clause_Chain (Target : Iir_Use_Clause;
                                   Chain : Iir_Use_Clause);
   function Get_Use_Clause_Chain (Target : Iir_Use_Clause)
                                 return Iir_Use_Clause;

   --  Selected name of an use_clause.
   --  Field: Field1
   procedure Set_Selected_Name (Target: Iir_Use_Clause; Name: Iir);
   function Get_Selected_Name (Target: Iir_Use_Clause) return Iir;

   --  Field: Field1
   procedure Set_Type_Declarator (Target: Iir; Decl: Iir);
   function Get_Type_Declarator (Target: Iir) return Iir;

   --  Field: Field3
   function Get_Enumeration_Literal_List
     (Def: Iir_Enumeration_Type_Definition) return Iir;

   --  Field: Field1
   function Get_Entity_Class_Entry_Chain
     (Group : Iir_Group_Template_Declaration)
     return Iir_Entity_Class;
   procedure Set_Entity_Class_Entry_Chain
     (Group : Iir_Group_Template_Declaration; Chain : Iir_Entity_Class);

   --  Field: Field1
   function Get_Group_Constituent_List (Group : Iir_Group_Declaration)
                                        return Iir_Group_Constituent_List;

   --  Field: Field2
   procedure Set_Primary_Unit (Target: Iir_Physical_Type_Definition;
                               Unit: Iir_Unit_Declaration);
   function Get_Primary_Unit (Target: Iir_Physical_Type_Definition)
                              return Iir_Unit_Declaration;

   --  Field: Field3
   procedure Set_Unit_Chain (Target: Iir_Physical_Type_Definition;
                             Chain : Iir);
   function Get_Unit_Chain (Target: Iir_Physical_Type_Definition)
                           return Iir;

   --  Field: Field2
   function Get_Multiplier (Target: Iir_Unit_Declaration) return Iir;
   procedure Set_Multiplier (Target: Iir_Unit_Declaration; Val: Iir);

   --  Get/Set the identifier of a declaration.
   --  Can also be used instead of get/set_label.
   --  Field: Ident
   procedure Set_Identifier (Target: Iir; Identifier: Name_Id);
   function Get_Identifier (Target: Iir) return Name_Id;

   --  Get/Set the visible flag of a declaration.
   --  The visible flag is true to make invalid the use of the identifier
   --  during its declaration.  It is set to false when the identifier is added
   --  to the name table, and set to true when the declaration is finished.
   --  Field: Flag4
   procedure Set_Visible_Flag (Target : Iir; Flag : Boolean);
   function Get_Visible_Flag (Target : Iir) return Boolean;

   --  Field: Field2
   function Get_Range_Constraint (Target : Iir) return Iir;
   procedure Set_Range_Constraint (Target : Iir; Constraint : Iir);

   --  Field: State1
   function Get_Direction (Decl: Iir) return Iir_Direction;
   procedure Set_Direction (Decl: Iir; Dir: Iir_Direction);

   --  Field: Field2
   function Get_Left_Limit (Decl: Iir_Range_Expression) return Iir;
   procedure Set_Left_Limit (Decl: Iir_Range_Expression; Limit: Iir);

   --  Field: Field3
   function Get_Right_Limit (Decl: Iir_Range_Expression) return Iir;
   procedure Set_Right_Limit (Decl: Iir_Range_Expression; Limit: Iir);

   --  Field: Field4
   function Get_Base_Type (Decl: Iir) return Iir;
   procedure Set_Base_Type (Decl: Iir; Base_Type: Iir);

   --  Field: Field5
   function Get_Resolution_Function (Decl: Iir) return Iir;
   procedure Set_Resolution_Function (Decl: Iir; Func: Iir);

   --  Field: Flag3
   function Get_Text_File_Flag (Atype : Iir_File_Type_Definition)
     return Boolean;
   procedure Set_Text_File_Flag (Atype : Iir_File_Type_Definition;
                                 Flag : Boolean);

   --  Field: Staticness_1
   procedure Set_Type_Staticness (Target: Iir; Static: Iir_Staticness);
   function Get_Type_Staticness (Target: Iir) return Iir_Staticness;

   --  Field: Field2
   function Get_Index_List (Decl: Iir) return Iir_List;
   procedure Set_Index_List (Decl: Iir; List : Iir_List);

   --  Field: Field3
   function Get_Element_Subtype (Decl: Iir) return Iir;
   procedure Set_Element_Subtype (Decl: Iir; Sub_Type: Iir);

   --  Field: Field2
   --  Chains of elements of a record.
   function Get_Element_Declaration_Chain (Decl: Iir_Record_Type_Definition)
                                          return Iir_Element_Declaration;
   procedure Set_Element_Declaration_Chain (Decl: Iir_Record_Type_Definition;
                                            Chain : Iir_Element_Declaration);

   --  Field: Nbr2
   --  Number of elements in the record.
   function Get_Number_Element_Declaration (Decl : Iir_Record_Type_Definition)
                                           return Iir_Index32;
   procedure Set_Number_Element_Declaration (Decl : Iir_Record_Type_Definition;
                                             Val : Iir_Index32);

   --  Field: Field2
   function Get_Designated_Type (Target: Iir) return Iir;
   procedure Set_Designated_Type (Target: Iir; Dtype: Iir);

   --  Field: Ident
   procedure Set_Label (Target: Iir; Label: Name_Id);
   function Get_Label (Target: Iir) return Name_Id;

   --  Field: Field2
   procedure Set_Target (Target: Iir; Atarget: Iir);
   function Get_Target (Target: Iir) return Iir;

   --  Field: Field3
   function Get_Waveform_Chain (Target: Iir) return Iir_Waveform_Element;
   procedure Set_Waveform_Chain (Target: Iir; Chain: Iir_Waveform_Element);

   --  Field: Field4
   function Get_Guard (Target: Iir) return Iir;
   procedure Set_Guard (Target: Iir; Guard: Iir);

   --  Field: Field7
   function Get_Delay_Mechanism (Target: Iir) return Iir_Delay_Mechanism;
   procedure Set_Delay_Mechanism
     (Target: Iir; Delay_Mechanism: Iir_Delay_Mechanism);

   --  Field: Field5
   function Get_Reject_Time_Expression (Target: Iir) return Iir;
   procedure Set_Reject_Time_Expression (Target: Iir; Expr: Iir);

   --  Field: Field4
   procedure Set_Sensitivity_List
     (Wait: Iir_Wait_Statement; List: Iir_Designator_List);
   function Get_Sensitivity_List (Target: Iir) return Iir_Designator_List;

   --  Field: Field2
   procedure Set_Condition_Clause (Wait: Iir_Wait_Statement; Cond: Iir);
   function Get_Condition_Clause (Wait: Iir_Wait_Statement) return Iir;

   --  Field: Field3
   procedure Set_Timeout_Clause (Wait: Iir_Wait_Statement; Timeout: Iir);
   function Get_Timeout_Clause (Wait: Iir_Wait_Statement) return Iir;

   --  If set, the concurrent statement is postponed.
   --  Field: Flag3
   procedure Set_Postponed_Flag (Target : Iir; Value : Boolean);
   function Get_Postponed_Flag (Target : Iir) return Boolean;

   --  Returns a list of signal or ports which are assigned in the current
   --  subprogram or process.
   --  Can return null_iir if there is no such assignment.
   --  Field: Field5
   function Get_Driver_List (Stmt: Iir) return Iir_Driver_List;
   procedure Set_Driver_List (Stmt: Iir; List: Iir_Driver_List);

   --  Returns the list of subprogram called in this subprogram or process.
   --  Note: implicit function (such as implicit operators) are omitted
   --  from this list, since the purpose of this list is to correctly set
   --  flags for side effects (passive_state, wait_state).
   --  Can return null_iir if there is no subprogram called.
   --  Field: Field7
   procedure Set_Callees_List (Proc : Iir; List : Iir_Callees_List);
   function Get_Callees_List (Proc : Iir) return Iir_Callees_List;

   --  Get/Set the passive flag of a process.
   --  TRUE if the process must be passive.
   --  FALSE if the process may be not passive.
   --  Field: Flag2
   procedure Set_Passive_Flag (Proc : Iir; Flag : Boolean);
   function Get_Passive_Flag (Proc : Iir) return Boolean;

   --  Get/Set the passive state of the current subprogram or process.
   --  FALSE if it contains a signal assignment in it or in any
   --   of the subprograms that may be called, directly or indirectly.
   --  TRUE if it contains no signal assignment or if all subprograms
   --   that may be called are passive.
   --  UNKNOWN if the passivity is unknown (generally because the
   --   subprogram body is not yet present).
   --  Field: State1
   procedure Set_Passive_State (Proc : Iir; State : Tri_State_Type);
   function Get_Passive_State (Proc : Iir) return Tri_State_Type;

   --  Get/Set the wait state of the current subprogram or process.
   --  TRUE if it contains a wait statement, either directly or
   --   indirectly.
   --  FALSE if it doesn't contain a wait statement.
   --  UNKNOWN if the wait status is not yet known.
   --  Field: State2
   procedure Set_Wait_State (Proc : Iir; State : Tri_State_Type);
   function Get_Wait_State (Proc : Iir) return Tri_State_Type;

   --  Get/Set the seen flag.
   --  Used when the graph of callees is walked, to avoid infinite loops, since
   --  the graph is not a DAG (there may be cycles).
   --  Field: Flag1
   procedure Set_Seen_Flag (Proc : Iir; Flag : Boolean);
   function Get_Seen_Flag (Proc : Iir) return Boolean;

   --  Get/Set the pure flag of a function.
   --  TRUE if the function is declared pure.
   --  FALSE if the function is declared impure.
   --  Field: Flag2
   procedure Set_Pure_Flag (Func : Iir; Flag : Boolean);
   function Get_Pure_Flag (Func : Iir) return Boolean;

   --  Get/Set the foreign flag of a declaration.
   --  TRUE if the declaration was decored with the std.foreign attribute.
   --  Field: Flag3
   procedure Set_Foreign_Flag (Decl : Iir; Flag : Boolean);
   function Get_Foreign_Flag (Decl : Iir) return Boolean;

   --  Get/Set the resolved flag of a subtype definition.
   --  A subtype definition may be resolved either because a
   --  resolution_function_name is present in the subtype_indication, or
   --  because all elements type are resolved.
   --  Field: Flag1
   procedure Set_Resolved_Flag (Atype : Iir; Flag : Boolean);
   function Get_Resolved_Flag (Atype : Iir) return Boolean;

   --  Get/Set the signal_type flag of a type/subtype definition.
   --  This flags indicates whether the type can be used as a signal type.
   --  Access types, file types and composite types whose a sub-element is
   --  an access type cannot be used as a signal type.
   --  Field: Flag2
   procedure Set_Signal_Type_Flag (Atype : Iir; Flag : Boolean);
   function Get_Signal_Type_Flag (Atype : Iir) return Boolean;

   --  Get/Set the purity status of a subprogram.
   --  FALSE if the subprogram performs directly or indirectly an impure
   --   action, as defined in LRM93 2.2
   --  TRUE if the subprogram as well as all of the subprograms it calls are
   --   pure.
   --  UNKNOWN in the case it is not known.
   --  Field: State3
   procedure Set_Purity_State (Proc : Iir; State : Tri_State_Type);
   function Get_Purity_State (Proc: Iir) return Tri_State_Type;

   --  Set during binding when DESIGN is added in a list of file to bind.
   --  Field: Flag3
   procedure Set_Elab_Flag (Design : Iir; Flag : Boolean);
   function Get_Elab_Flag (Design : Iir) return Boolean;

   --  Condition of an assertion.
   --  Field: Field2
   procedure Set_Assertion_Condition (Target: Iir; Cond: Iir);
   function Get_Assertion_Condition (Target: Iir) return Iir;

   --  Report expression of an assertion or report statement.
   --  Field: Field3
   procedure Set_Report_Expression (Target: Iir; Expr: Iir);
   function Get_Report_Expression (Target: Iir) return Iir;

   --  Severity expression of an assertion or report statement.
   --  Field: Field4
   procedure Set_Severity_Expression (Target: Iir; Expr: Iir);
   function Get_Severity_Expression (Target: Iir) return Iir;

   --  Instantiated unit of a component instantiation statement.
   --  Field: Field2
   procedure Set_Instantiated_Unit (Target: Iir; Unit: Iir);
   function Get_Instantiated_Unit (Target: Iir) return Iir;

   --  Generic map aspect list.
   --  Field: Field5
   procedure Set_Generic_Map_Aspect_Chain (Target: Iir; Generics: Iir);
   function Get_Generic_Map_Aspect_Chain (Target: Iir) return Iir;

   --  Port map aspect list.
   --  Field: Field3
   procedure Set_Port_Map_Aspect_Chain (Target: Iir; Port: Iir);
   function Get_Port_Map_Aspect_Chain (Target: Iir) return Iir;

   --  Configuration of an entity_aspect_configuration.
   --  Field: Field1
   procedure Set_Configuration (Target: Iir; Conf : Iir);
   function Get_Configuration (Target : Iir) return Iir;

   --  Configuration for a component_instantiation_statement.
   --  Field: Field1
   procedure Set_Component_Configuration
     (Target : Iir_Component_Instantiation_Statement; Conf : Iir);
   function Get_Component_Configuration
     (Target : Iir_Component_Instantiation_Statement) return Iir;

   --  Set/Get the default binding indication of a configuration specification
   --  or a component configuration.
   --  Field: Field4
   procedure Set_Default_Binding_Indication (Target : Iir; Conf : Iir);
   function Get_Default_Binding_Indication (Target : Iir) return Iir;

   --  Set/Get the default configuration of an architecture.
   --  Field: Field4
   procedure Set_Default_Configuration_Declaration
     (Target : Iir_Architecture_Declaration; Conf : Iir);
   function Get_Default_Configuration_Declaration
     (Target : Iir_Architecture_Declaration)
     return Iir;

   --  Expression for an various nodes.
   --  Field: Field3
   procedure Set_Expression (Target: Iir; Expr: Iir);
   function Get_Expression (Target: Iir) return Iir;

   --  Field: Field1
   function Get_Selected_Waveform_Chain (Target: Iir) return Iir;
   procedure Set_Selected_Waveform_Chain (Target : Iir; Chain : Iir);

   --  Field: Field3
   function Get_Conditional_Waveform_Chain
     (Target: Iir_Concurrent_Conditional_Signal_Assignment)
     return Iir_Conditional_Waveform;
   procedure Set_Conditional_Waveform_Chain
     (Target: Iir_Concurrent_Conditional_Signal_Assignment;
      Chain : Iir_Conditional_Waveform);

   --  Expression defining the value of the implicit guard signal.
   --  Field: Field2
   procedure Set_Guard_Expression (Target: Iir_Guard_Signal_Declaration;
                                   Expr: Iir);
   function Get_Guard_Expression (Target: Iir_Guard_Signal_Declaration)
     return Iir;

   --  The declaration (if any) of the implicit guard signal of a block
   --  statement.
   --  Field: Field1
   function Get_Guard_Decl (Target: Iir_Block_Statement)
     return Iir_Guard_Signal_Declaration;
   procedure Set_Guard_Decl (Target: Iir_Block_Statement;
                             Decl: Iir_Guard_Signal_Declaration);

   --  Field: Field5
   procedure Set_Guard_Sensitivity_List
     (Guard: Iir_Guard_Signal_Declaration; List: Iir_Designator_List);
   function Get_Guard_Sensitivity_List (Guard : Iir_Guard_Signal_Declaration)
     return Iir_Designator_List;

   --  Block_Configuration that applies to this block statement.
   --  Field: Field4
   procedure Set_Block_Block_Configuration
     (Block : Iir_Block_Statement; Conf : Iir_Block_Configuration);
   function Get_Block_Block_Configuration (Block : Iir_Block_Statement)
     return Iir_Block_Configuration;

   --  Field: Field7
   procedure Set_Block_Header
     (Block: Iir_Block_Statement; Header: Iir_Block_Header);
   function Get_Block_Header (Block: Iir_Block_Statement)
     return Iir_Block_Header;

   --  Get/Set the block_configuration (there may be several
   --  block_configuration through the use of prev_configuration singly linked
   --  list) that apply to this generate statement.
   --  Field: Field1
   procedure Set_Generate_Block_Configuration
     (Block : Iir_Generate_Statement; Conf : Iir_Block_Configuration);
   function Get_Generate_Block_Configuration (Block : Iir_Generate_Statement)
     return Iir_Block_Configuration;

   --  Field: Field5
   procedure Set_Generation_Scheme
     (Block : Iir_Generate_Statement; Scheme : Iir);
   function Get_Generation_Scheme (Block : Iir_Generate_Statement) return Iir;

   --  Condition of a conditionam_waveform, if_statement, elsif,
   --  while_loop_statement, next_statement or exit_statement.
   --  Field: Field1
   procedure Set_Condition (Target: Iir; Condition: Iir);
   function Get_Condition (Target: Iir) return Iir;

   --  Field: Field3
   procedure Set_Else_Clause (Target: Iir; Clause: Iir_Elsif);
   function Get_Else_Clause (Target: Iir) return Iir_Elsif;

   --  Iterator of a for_loop_statement.
   --  Field: Field1
   procedure Set_Iterator_Scheme
     (Target: Iir_For_Loop_Statement; Iterator: Iir);
   function Get_Iterator_Scheme (Target: Iir_For_Loop_Statement) return Iir;

   --  Get/Set the statement in which TARGET appears.  This is used to check
   --  if next/exit is in a loop.
   --  Field: Parent
   procedure Set_Parent (Target : Iir; Parent : Iir);
   function Get_Parent (Target : Iir) return Iir;

   --  Loop label for an exit_statement or next_statement.
   --  Field: Field2
   function Get_Loop (Target: Iir) return Iir;
   procedure Set_Loop (Target: Iir; Stmt: Iir);

   --  Component name for a component_configuration or
   --  a configuration_specification.
   --  Field: Field4
   procedure Set_Component_Name (Target: Iir; Name: Iir);
   function Get_Component_Name (Target: Iir) return Iir;

   --  Field: Field1
   function Get_Instantiation_List (Target: Iir) return Iir;
   procedure Set_Instantiation_List (Target: Iir; List : Iir);

   --  Field: Field4
   procedure Set_Entity_Aspect (Target: Iir_Binding_Indication; Entity: Iir);
   function Get_Entity_Aspect (Target: Iir_Binding_Indication) return Iir;

   --  Field: Field1
   procedure Set_Default_Entity_Aspect (Target : Iir_Binding_Indication;
                                        Aspect : Iir);
   function Get_Default_Entity_Aspect (Target : Iir_Binding_Indication)
     return Iir;

   --  Field: Field2
   procedure Set_Default_Generic_Map_Aspect_Chain
     (Target : Iir_Binding_Indication; Chain : Iir);
   function Get_Default_Generic_Map_Aspect_Chain
     (Target : Iir_Binding_Indication)
     return Iir;

   --  Field: Field6
   procedure Set_Default_Port_Map_Aspect_Chain
     (Target : Iir_Binding_Indication; Chain : Iir);
   function Get_Default_Port_Map_Aspect_Chain (Target : Iir_Binding_Indication)
     return Iir;

   --  Field: Field3
   procedure Set_Binding_Indication (Target : Iir; Binding : Iir);
   function Get_Binding_Indication (Target : Iir) return Iir;

   --  The named entity designated by a name.
   --  Field: Field5
   procedure Set_Named_Entity (Target : Iir; Val : Iir);
   function Get_Named_Entity (Target : Iir) return Iir;

   --  Expression staticness, defined by rules of LRM 7.4
   --  Field: Staticness_1
   procedure Set_Expr_Staticness (Target: Iir; Static: Iir_Staticness);
   function Get_Expr_Staticness (Target: Iir) return Iir_Staticness;

   --  Operand of a monadic operator.
   --  Field: Field2
   procedure Set_Operand (Target: Iir; An_Iir: Iir);
   function Get_Operand (Target: Iir) return Iir;

   --  Left operand of a dyadic operator.
   --  Field: Field2
   procedure Set_Left (Target: Iir; An_Iir: Iir);
   function Get_Left (Target: Iir) return Iir;

   --  Right operand of a dyadic operator.
   --  Field: Field4
   procedure Set_Right (Target: Iir; An_Iir: Iir);
   function Get_Right (Target: Iir) return Iir;

   --  Field: Field2
   procedure Set_Name (Target: Iir; Name: Iir);
   function Get_Name (Target: Iir) return Iir;

   --  Staticness of a name, according to rules of LRM 6.1
   --  Field: Staticness_2
   procedure Set_Name_Staticness (Target: Iir; Static: Iir_Staticness);
   function Get_Name_Staticness (Target: Iir) return Iir_Staticness;

   --  Prefix of a name.
   --  Field: Field3
   procedure Set_Prefix (Target: Iir; Prefix: Iir);
   function Get_Prefix (Target: Iir) return Iir;

   --  Suffix of a slice or attribute.
   --  Field: Field2
   procedure Set_Suffix (Target: Iir; Suffix: Iir);
   function Get_Suffix (Target: Iir) return Iir;

   --  Parameter of an attribute.
   --  Field: Field2
   procedure Set_Parameter (Target: Iir; Param: Iir);
   function Get_Parameter (Target: Iir) return Iir;

   --  Type of the actual for an association by individual.
   --  Unless the formal is an unconstrained array type, this is the same as
   --  the formal type.
   --  Field: Field2
   procedure Set_Actual_Type (Target : Iir; Atype : Iir);
   function Get_Actual_Type (Target : Iir) return Iir;

   --  List of individual associations for association_element_by_individual.
   --  Associations for parenthesis_name.
   --  Field: Field4
   procedure Set_Association_Chain (Target: Iir; Chain: Iir);
   function Get_Association_Chain (Target: Iir) return Iir;

   --  List of individual associations for association_element_by_individual.
   --  Field: Field4
   procedure Set_Individual_Association_Chain
     (Target: Iir_Association_Element_By_Individual; Chain : Iir);
   function Get_Individual_Association_Chain
     (Target: Iir_Association_Element_By_Individual) return Iir;

   --  Get/Set info for the aggregate.
   --  There is one aggregate_info for for each dimension.
   --  Field: Field3
   procedure Set_Aggregate_Info (Target : Iir_Aggregate;
                                 Info : Iir_Aggregate_Info);
   function Get_Aggregate_Info (Target : Iir_Aggregate)
                               return Iir_Aggregate_Info;

   --  Get/Set the info node for the next dimension.
   --  Field: Field1
   procedure Set_Sub_Aggregate_Info (Target : Iir_Aggregate_Info;
                                     Info : Iir_Aggregate_Info);
   function Get_Sub_Aggregate_Info (Target : Iir_Aggregate_Info)
                                   return Iir_Aggregate_Info;

   --  TRUE when the length of the aggregate is not locally static.
   --  Field: Flag3
   procedure Set_Aggr_Dynamic_Flag (Target : Iir_Aggregate_Info;
                                    Val : Boolean);
   function Get_Aggr_Dynamic_Flag (Target : Iir_Aggregate_Info)
                                  return Boolean;

   --  Get/Set the maximum number of elements for the lowest dimension of
   --  the aggregate or for the current dimension of a sub-aggregate.
   --  The real number of elements may be greater than this number if there
   --  is an 'other' choice.
   --  Field: Nbr2
   procedure Set_Aggr_Max_Length (Target : Iir_Aggregate_Info; Nbr : Integer);
   function Get_Aggr_Max_Length (Target : Iir_Aggregate_Info) return Integer;

   --  Highest index choice, if any.
   --  Field: Field2
   procedure Set_Aggr_Low_Limit (Target : Iir_Aggregate_Info; Limit : Iir);
   function Get_Aggr_Low_Limit (Target : Iir_Aggregate_Info) return Iir;

   --  Highest index choice, if any.
   --  Field: Field3
   procedure Set_Aggr_High_Limit (Target : Iir_Aggregate_Info; Limit : Iir);
   function Get_Aggr_High_Limit (Target : Iir_Aggregate_Info) return Iir;

   --  True if the aggregate has an 'others' choice.
   --  Field: Flag2
   procedure Set_Aggr_Others_Flag (Target : Iir_Aggregate_Info; Val : Boolean);
   function Get_Aggr_Others_Flag (Target : Iir_Aggregate_Info) return Boolean;

   --  True if the aggregate have named associations.
   --  Field: Flag4
   procedure Set_Aggr_Named_Flag (Target : Iir_Aggregate_Info; Val : Boolean);
   function Get_Aggr_Named_Flag (Target : Iir_Aggregate_Info) return Boolean;

   --  Chain of choices.
   --  Field: Field4
   function Get_Association_Choices_Chain (Target: Iir) return Iir;
   procedure Set_Association_Choices_Chain (Target : Iir; Chain : Iir);

   --  Chain of choices.
   --  Field: Field4
   function Get_Case_Statement_Alternative_Chain (Target: Iir_Case_Statement)
                                                 return Iir;
   procedure Set_Case_Statement_Alternative_Chain
     (Target : Iir_Case_Statement; Chain : Iir);

   --  Staticness of the choice.
   --  Field: Staticness_2
   function Get_Choice_Staticness (Target : Iir) return Iir_Staticness;
   procedure Set_Choice_Staticness (Target: Iir; Staticness : Iir_Staticness);

   --  Subprogram to be called by a procedure, function call or operator.
   --  Field: Field3
   procedure Set_Implementation (Target: Iir; Decl: Iir);
   function Get_Implementation (Target: Iir) return Iir;

   --  Paramater associations for procedure and function call.
   --  Field: Field2
   procedure Set_Parameter_Association_Chain (Target: Iir; Chain : Iir);
   function Get_Parameter_Association_Chain (Target: Iir) return Iir;

   --  The type_mark that appeared in the subtype indication.
   --  May be null_iir if there is no type mark (as in an iterator).
   --  May differ from base_type, if the type_mark is a subtype_name.
   --  Field: Field6
   procedure Set_Type_Mark (Target: Iir; Mark: Iir);
   function Get_Type_Mark (Target: Iir) return Iir;

   --  Get/set the lexical layout of an interface.
   --  Field: Odigit_1
   procedure Set_Lexical_Layout
     (Target : Iir; Layout : Iir_Lexical_Layout_Type);
   function Get_Lexical_Layout (Target : Iir) return Iir_Lexical_Layout_Type;

   --  List of use (designated type of access types) of an incomplete type
   --  definition.  The purpose is to complete the uses with the full type
   --  definition.
   --  Field: Field2
   function Get_Incomplete_Type_List (Target : Iir) return Iir_List;

   --  This flag is set on a signal_declaration, when a disconnection
   --  specification applies to the signal (or a subelement of it).
   --  This is used to check 'others' and 'all' designators.
   --  Field: Flag1
   function Get_Has_Disconnect_Flag (Target : Iir) return Boolean;
   procedure Set_Has_Disconnect_Flag (Target : Iir; Val : Boolean);

   --  This flag is set is code being analyzed is textually within TARGET.
   --  This is used for selected by name rule.
   --  Field: Flag5
   function Get_Is_Within_Flag (Target : Iir) return Boolean;
   procedure Set_Is_Within_Flag (Target : Iir; Val : Boolean);

   --  List of type_mark for an Iir_Kind_Signature
   --  Field: Field3
   function Get_Type_Marks_List (Target : Iir_Signature) return Iir_List;
   procedure Set_Type_Marks_List (Target : Iir_Signature; List : Iir_List);

   --  Field: Field4
   function Get_Signature (Target : Iir) return Iir_Signature;
   procedure Set_Signature (Target : Iir; Value : Iir);
private
   --  A simple type that needs only 2 bits.
   type Bit2_Type is range 0 .. 2 ** 2 - 1;
   type Bit3_Type is range 0 .. 2 ** 3 - 1;

   Null_Iir : constant Iir := null;
   Null_Iir_List : constant Iir_List := null;
   Error_Iir : constant Iir := null;

   type Iir_Node (Kind: Iir_Kind := Iir_Kind_Error) is record

      --  Kind field use 8 bits.
      --  So, on 32 bits systems, there are 24 bits left.
      --  + 8 (8 * 1)
      --  + 10 (5 * 2)
      --  + 6 (2 * 3)
      --  = 24

      --  Usages of Flag1:
      --  seen_flag for iir_kind_process_statement
      --  seen_flag for iir_kind_sensitized_process_statement
      --  seen_flag for iir_kinds_procedure_specification
      --  seen_flag for iir_kinds_function_specification
      --  seen_flag for iir_kind_design_file
      --  deferred_declaration_flag for iir_kind_constant_declaration
      --  loaded_flag for iir_kind_design_unit
      --  resolved_flag for iir_kinds_type_definition
      --  need_body for iir_kind_package_declaration
      --  whole_association_flag for iir_kind_association_element_by_expression
      --  has_disconnect_flag for iir_kind_signal_declaration
      Flag1 : Boolean := False;

      --  Usages of Flag2:
      --  pure_flag for iir_kinds_function_specification
      --  passive_flag for iir_kinds_process_statement
      --  shared_flag for iir_kind_variable_declaration
      --  aggr_others_flag for iir_kind_aggregate_info
      --  signal_type_flag for iir_kinds_type_definition
      Flag2 : Boolean := False;

      --  Usages of Flag3:
      --  (postponed_flag for iir_kinds_process_statement)
      --  elab_flag for iir_kind_design_file
      --  elab_flag for iir_kind_design_unit
      --  dynamic_flag for iir_kind_aggregate_info
      --  text_file_flag for iir_kind_file_type_definition
      --  foreign_flag for iir_kind_architecture_declaration
      --  foreign_flag for iir_kinds_function_specification
      --  foreign_flag for iir_kinds_procedure_specification
      Flag3 : Boolean := False;

      --  Usages of Flag4:
      --  visible_flag for iir_kind_type_declaration
      --  aggr_named_flag for iir_kind_aggregate_info
      Flag4 : Boolean := False;

      --  Usages of Flag5:
      --  is_within_flag for named entities
      Flag5 : Boolean := False;

      --  Usages of Flag6:
      Flag6 : Boolean := False;

      --  Usages of Flag7:
      Flag7 : Boolean := False;

      --  Usages of Flag8:
      Flag8 : Boolean := False;

      --  2bits fields (4 -> 8 bits)
      --  Usages of State1:
      --  passive_state for iir_kind_process_statement
      --  passive_state for iir_kind_sensitized_process_statement
      --  passive_state for iir_kinds_procedure_specification
      --  passive_state for iir_kinds_function_specification
      --  signal_kind for iir_kind_signal_declaration
      --  signal_kind for iir_kind_guard_signal_declaration
      --  signal_kind for iir_kind_signal_interface_declaration
      --  direction for iir_kind_range_expression
      --  direction for iir_kind_file_declaration
      --  guarded_target_flag for iir_kind_concurrent_conditional_signal_assign
      --  guarded_target_flag for iir_kind_selected_conditional_signal_assign
      --  guarded_target_flag for iir_kind_signal_assignment_statement
      State1 : Bit2_Type := 0;

      --  Usages of State2:
      --  wait_state for iir_kind_process_statement
      --  wait_state for iir_kind_sensitized_process_statement
      --  wait_state for iir_kinds_procedure_specification
      --  wait_state for iir_kinds_function_specification
      State2 : Tri_State_Type := Unknown;

      --  Usages of State3:
      --  purity_state for iir_kind_process_statement
      --  purity_state for iir_kind_sensitized_process_statement
      --  purity_state for iir_kinds_procedure_specification
      --  purity_state for iir_kinds_function_specification
      State3 : Tri_State_Type := Unknown;

      -- expr_staticness for iir_kind_string_literal
      -- expr_staticness for iir_kind_bit_string_literal
      -- expr_staticness for iir_kind_integer_literal
      -- expr_staticness for iir_kind_floating_point_literal
      -- expr_staticness for iir_kind_physical_int_literal
      -- expr_staticness for iir_kind_physical_fp_literal
      -- expr_staticness for iir_kind_enumeration_literal
      -- expr_staticness for iir_kind_monadic_operator
      -- expr_staticness for iir_kind_dyadic_operator
      -- expr_staticness for iir_kinds_name
      -- expr_staticness for iir_kinds_alias_declaration
      -- expr_staticness for iir_kind_constant_declaration
      -- expr_staticness for iir_kind_iterator_declaration
      -- expr_staticness for iir_kind_constant_interface_declaration
      -- expr_staticness for iir_kind_aggregate
      -- expr_staticness for iir_kind_qualified_expression
      -- expr_staticness for iir_kind_type_conversion
      -- expr_staticness for iir_kind_length_array_attribute
      -- expr_staticness for iir_kind_low_type_attribute
      -- expr_staticness for iir_kind_high_type_attribute
      -- expr_staticness for iir_kind_left_type_attribute
      -- expr_staticness for iir_kind_right_type_attribute
      -- expr_staticness for iir_kind_pos_attribute
      -- expr_staticness for iir_kind_val_attribute
      -- expr_staticness for iir_kind_event_attribute
      -- expr_staticness for iir_kind_last_value_attribute
      -- expr_staticness for iir_kind_last_active_attribute
      -- expr_staticness for iir_kind_active_attribute
      -- expr_staticness for iir_kind_range_expression
      -- expr_staticness for iir_kind_selected_element
      -- expr_staticness for iir_kind_function_call
      -- expr_staticness for iir_kind_attribute_value
      -- expr_staticness for iir_kind_signal_declaration
      -- expr_staticness for iir_kind_guard_signal_declaration
      -- expr_staticness for iir_kind_variable_declaration
      -- expr_staticness for iir_kind_file_declaration
      -- expr_staticness for iir_kinds_discrete_type_attribute
      -- type_staticness for iir_kinds_type_and_subtype_definition
      Staticness_1 : Iir_Staticness := Unknown;

      -- name_staticness for iir_kinds_name
      -- name_staticness for iir_kind_object_alias_declaration
      -- name_staticness for iir_kind_selected_element
      -- name_staticness for iir_kind_selected_by_all_name
      -- choice_staticness for iir_kind_choice_by_range
      -- choice_staticness for iir_kind_choice_by_expression
      Staticness_2 : Iir_Staticness := Unknown;

      -- 3bits fields (1 -> 3 bits)
      --  Usages of odigit_1:
      --  lexical_layout for iir_kinds_interface_declaration
      Odigit_1 : Bit3_Type := 0;

      --  Usage of odigit_2:
      -- iir_mode for iir_kind_signal_interface_declaration
      -- iir_mode for iir_kind_constant_interface_declaration
      -- iir_mode for iir_kind_variable_interface_declaration
      -- iir_mode for iir_kind_file_interface_declaration
      Odigit_2 : Bit3_Type := 0;

      -- Location.
      Location: Location_Type := Location_Nil;

      Back_End_Info: System.Address := System.Null_Address;

      -- Usages of identifier:
      -- identifier for iir_kind_type_declaration
      -- identifier for iir_kind_subtype_declaration
      -- identifier for iir_kind_architecture_declaration
      -- identifier for iir_kind_configuration_declaration
      -- identifier for iir_kind_component_declaration
      -- identifier for iir_kind_entity_declaration
      -- identifier for iir_kind_signal_interface_declaration
      -- identifier for iir_kind_signal_declaration
      -- identifier for iir_kind_guard_signal_declaration
      -- identifier for iir_kind_constant_interface_declaration
      -- identifier for iir_kind_constant_declaration
      -- identifier for iir_kind_iterator_declaration
      -- identifier for iir_kind_variable_interface_declaration
      -- identifier for iir_kind_variable_declaration
      -- identifier for iir_kind_file_interface_declaration
      -- identifier for iir_kind_file_declaration
      -- identifier for iir_kind_element_declaration
      -- identifier for iir_kind_attribute_declaration
      -- identifier for iir_kind_enumeration_literal
      -- identifier for iir_kind_unit_declaration
      -- identifier for iir_kind_function_Specification
      -- identifier for iir_kind_function_Spec_Body
      -- identifier for iir_kind_implicit_function_declaration
      -- identifier for iir_kind_procedure_Specification
      -- identifier for iir_kind_procedure_Spec_Body
      -- identifier for iir_kind_implicit_procedure_declaration
      -- identifier for iir_kind_package_declaration
      -- identifier for iir_kind_package_body
      -- identifier for iir_kind_library_declaration
      -- identifier for iir_kind_library_clause
      -- identifier for iir_kind_object_alias_declaration
      -- identifier for iir_kind_selected_name
      -- identifier for iir_kind_simple_name
      -- identifier for iir_kind_string_literal
      -- identifier for iir_kind_bit_string_literal
      -- identifier for iir_kind_character_literal
      -- identifier for iir_kind_group_template_declaration
      -- identifier for iir_kind_group_declaration
      -- identifier for iir_kind_user_attribute
      -- identifier for iir_kind_attribute
      -- label for iir_kind_sensitized_process_statement
      -- label for iir_kind_process_statement
      -- label for Iir_Kind_Component_Instantiation_Statement
      -- label for iir_kind_concurrent_assertion_statement
      -- label for iir_kind_concurrent_conditional_signal_assignment
      -- label for iir_kind_concurrent_selected_signal_assignment
      -- label for iir_kind_concurrent_procedure_call_statement
      -- label for iir_kind_block_statement
      -- label for iir_kinds_sequential_statement
      -- label for iir_kind_while_loop_statement
      Identifier: Name_Id := Null_Identifier;

      Pad_0 : Boolean := False;

      -- attribute_value_list for iir_kind_entity_declaration
      -- attribute_value_list for iir_kind_architecture_declaration
      -- attribute_value_list for iir_kind_configuration_declaration
      -- attribute_value_list for iir_kind_function_specification
      -- attribute_value_list for iir_kind_function_spec_body
      -- attribute_value_list for iir_kind_procedure_specification
      -- attribute_value_list for iir_kind_procedure_spec_body
      -- attribute_value_list for iir_kind_implicit_function_declaration
      -- attribute_value_list for iir_kind_implicit_procedure_declaration
      -- attribute_value_list for iir_kind_package_declaration
      -- attribute_value_list for iir_kind_type_declaration
      -- attribute_value_list for iir_kind_subtype_declaration
      -- attribute_value_list for iir_kind_signal_declaration
      -- attribute_value_list for iir_kind_guard_signal_declaration
      -- attribute_value_list for iir_kind_variable_declaration
      -- attribute_value_list for iir_kind_constant_declaration
      -- attribute_value_list for iir_kind_iterator_declaration
      -- attribute_value_list for iir_kind_file_declaration
      -- attribute_value_list for iir_kind_signal_interface_declaration
      -- attribute_value_list for iir_kind_variable_interface_declaration
      -- attribute_value_list for iir_kind_constant_interface_declaration
      -- attribute_value_list for iir_kind_file_interface_declaration
      -- attribute_value_list for iir_kind_component_declaration
      -- attribute_value_list for iir_kind_group_declaration
      -- attribute_value_list for iir_kind_enumeration_literal
      -- attribute_value_list for iir_kind_unit_declaration
      -- attribute_value_list for iir_kind_concurrent_conditional_signal_ass.
      -- attribute_value_list for iir_kind_concurrent_selected_signal_assign.
      -- attribute_value_list for iir_kind_sensitized_process_statement
      -- attribute_value_list for iir_kind_process_statement
      -- attribute_value_list for iir_kind_concurrent_assertion_statement
      -- attribute_value_list for iir_kind_component_instantiation_statement
      -- attribute_value_list for iir_kind_concurrent_procedure_call_statement
      -- attribute_value_list for iir_kind_block_statement
      -- attribute_value_list for iir_kind_if_statement
      -- attribute_value_list for iir_kind_for_loop_statement
      -- attribute_value_list for iir_kind_while_loop_statement
      -- attribute_value_list for iir_kind_next_statement
      -- attribute_value_list for iir_kind_exit_statement
      -- attribute_value_list for iir_kind_signal_assignment_statement
      -- attribute_value_list for iir_kind_variable_assignment_statement
      -- attribute_value_list for iir_kind_assertion_statement
      -- attribute_value_list for iir_kind_report_statement
      -- attribute_value_list for iir_kind_wait_statement
      -- attribute_value_list for iir_kind_return_statement
      -- attribute_value_list for iir_kind_case_statement
      -- attribute_value_list for iir_kind_procedure_call
      -- attribute_value_list for iir_kind_null_statement
      -- attribute_value_list for iir_kind_attribute_specification
      Attr : Iir := Null_Iir;

      --  The parent node.
      -- parent for iir_kind_if_statement
      -- parent for iir_kind_elsif_statement
      -- parent for iir_kind_for_loop_statement
      -- parent for iir_kind_while_loop_statement
      -- parent for iir_kind_case_statement
      -- parent for iir_kind_exit_statement
      -- parent for iir_kind_next_statement
      -- parent (library_declaration) for iir_kind_design_file
      -- parent (design_unit_list) for iir_kind_design_file
      -- interface_parent for iir_kind_signal_interface_declaration
      -- interface_parent for iir_kind_constant_interface_declaration
      -- interface_parent for iir_kind_variable_interface_declaration
      -- interface_parent for iir_kind_file_interface_declaration
      Parent : Iir := Null_Iir;

      -- Usages of Chain:
      Chain : Iir := Null_Iir;

      -- usages of field1:
      -- type for iir_kind_character_literal
      -- type for iir_kind_type_computed_literal
      -- type for iir_kind_integer_literal
      -- type for iir_kind_floating_point_literal
      -- type for iir_type_declaration.
      -- type for iir_subtype_declaration.
      -- type for iir_kind_identifier
      -- type for iir_kind_string_literal
      -- type for iir_kind_bit_string_literal
      -- type for iir_kind_base_attribute
      -- list_element for iir_kinds_list
      -- port_chain for iir_kind_entity_declaration
      -- port_chain for iir_kind_component_declaration
      -- port_chain for iir_kind_block_header
      -- entity for iir_kind_architecture_declaration
      -- entity for iir_kind_configuration_declaration
      -- entity for iir_kind_entity_aspect_entity
      -- package for iir_kind_package_body
      -- primary_units(iir_library_unit_list) for iir_kind_library_declaration
      -- selected_name for iir_kind_use_clause
      -- type_declaration for iir_kinds_type_definition
      -- type_definition for iir_kind_signal_declaration
      -- type_definition for iir_kind_guard_signal_declaration
      -- type_definition for iir_kind_signal_interface_declaration.
      -- type_definition for iir_kind_variable_declaration
      -- type_definition for iir_kind_variable_interface_declaration.
      -- type_definition for iir_kind_constant_declaration
      -- type_definition for iir_kind_iterator_declaration
      -- type_definition for iir_kind_constant_interface_declaration.
      -- type_definition for iir_kind_file_declaration
      -- type_definition for iir_kind_file_interface_declaration.
      -- type_definition for iir_kind_enumeration_literal
      -- type_definition for iir_kind_unit_declaration
      -- type_definition for iir_kind_component_port
      -- type_definition for iir_kind_element_declaration
      -- type_definition for iir_kinds_attribute_declaration
      -- type_definition for iir_kinds_attribute
      -- type_definition for iir_kinds_name
      -- type_definition for iir_kind_return_statement
      -- type_definition for iir_kind_aggregate
      -- type_definition for iir_kind_physical_int_literal
      -- type_definition for iir_kind_physical_fp_literal
      -- type_definition for iir_kind_object_alias_declaration
      -- type_definition for iir_kind_null_literal
      -- type_definition for iir_kind_qualified_expression
      -- type_definition for iir_kind_type_conversion
      -- type_definition for iir_kind_function_call
      -- type_definition for iir_kind_allocator_by_expression
      -- type_definition for iir_kind_allocator_by_subtype
      -- type_definition for iir_kind_attribute_value
      -- type_definition for iir_kind_selected_element
      -- type_definition for iir_kind_implicit_dereference.
      -- type_definition for iir_kind_disconnection_specification
      -- type_definition for iir_kinds_monadic_operator
      -- type_definition for iir_kinds_dyadic_operator
      -- null_iir for iir_kind_signal_assignment_statement
      -- null_iir for iir_kind_variable_assignment_statement
      -- we_value for iir_kind_waveform_element
      -- condition for iir_kind_conditional_waveform
      -- condition for iir_kind_if_statement
      -- condition for iir_kind_elsif
      -- condition for iir_kind_while_loop_statement
      -- condition for iir_kind_next_statement
      -- condition for iir_kind_exit_statement
      -- design_unit_chain for iir_kind_design_file
      -- formal for iir_kinds_association_element
      -- iterator_scheme for iir_kind_for_loop_statement
      -- associated for iir_kinds_association_by_choice
      -- context_items for iir_kind_design_unit
      -- design_file_chain for iir_kind_library_declaration
      -- proxy for iir_kind_proxy
      -- selected_waveform_l for iir_kind_concurrent_selected_signal_assignment
      -- block_specification for iir_kind_block_configuration
      -- instantiation_list for iir_kind_component_configuration
      -- instantiation_list for iir_kind_configuration_specification
      -- component_configuration for iir_kind_component_instantiation_statement
      -- configuration for iir_kind_entity_aspect_configuration
      -- guard_decl for iir_kind_block_statement
      -- entity_class_entry_chain for iir_kind_group_template_declaration
      -- group_constituent_chain for iir_kind_group_declaration
      -- entity_name_list for iir_kind_attribute_specification
      -- generate_block_configuration for iir_kind_generate_statement
      -- type_declarator for Iir_Kind_Enumeration_Type_Definition
      -- type_declarator for Iir_Kind_Enumeration_Subtype_Definition
      -- type_declarator for Iir_Kind_Integer_Type_Definition
      -- type_declarator for Iir_Kind_Integer_Subtype_Definition
      -- type_declarator for Iir_Kind_Floating_Type_Definition
      -- type_declarator for Iir_Kind_Floating_Subtype_Definition
      -- type_declarator for Iir_Kind_Physical_Type_Definition
      -- type_declarator for Iir_Kind_Physical_Subtype_Definition
      -- type_declarator for Iir_Kind_Record_Type_Definition
      -- type_declarator for Iir_Kind_Record_Subtype_Definition
      -- type_declarator for Iir_Kind_Array_Type_Definition
      -- type_declarator for Iir_Kind_Array_Subtype_Definition
      -- type_declarator for Iir_Kind_Unconstrained_Array_Subtype_Definition
      -- type_declarator for Iir_Kind_Access_Type_Definition
      -- type_declarator for Iir_Kind_Access_Subtype_Definition
      -- type_declarator for Iir_Kind_Incomplete_Type_Definition
      -- type_declarator for Iir_Kind_File_Type_Definition
      -- return_type for iir_kind_function_specification
      -- return_type for iir_kind_function_spec_body
      -- return_type for iir_kind_implicit_function_declaration
      -- default_entity_aspect for iir_kind_binding_indication
      -- sub_aggregate_info for iir_kind_aggregate_info
      Field1: Iir := Null_Iir;

      -- usages of field2:
      -- concurrent_statement_list for iir_kind_architecture_declaration
      -- concurrent_statement_list for iir_kind_block_statement
      -- concurrent_statement_list for iir_kind_entity_declaration
      -- concurrent_statement_list for iir_kind_generate_statement
      -- block_configuration for iir_kind_configuration_declaration
      -- block_configuration for iir_kind_component_configuration
      -- subprogram_body for iir_kind_function_specification
      -- subprogram_body for iir_kind_procedure_specification
      -- range_constraint for iir_kind_integer_subtype_definition
      -- range_constraint for iir_kind_floating_subtype_definition
      -- range_constraint for iir_kind_subtype_definition
      -- range_constraint for iir_kind_enumeration_subtype_definition
      -- range_constraint for iir_kind_physical_subtype_definition
      -- range_constraint for iir_kind_enumeration_type_definition
      -- left_limit for iir_kind_range_expression
      -- designated_type for iir_kind_access_type_definition
      -- index_subtype for iir_array_type_definition
      -- index_subtype for iir_array_subtype_definition
      -- suffix for iir_kinds_attribute
      -- suffix for iir_kind_user_attribute
      -- suffix for iir_kind_slice_name
      -- selected_element for iir_kind_selected_element
      -- parameter for iir_kind_val_attribute
      -- parameter for iir_kind_pos_attribute
      -- parameter for iir_kind_delayed_attribute
      -- parameter for iir_kind_stable_attribute
      -- parameter for iir_kind_quiet_attribute
      -- parameter for iir_kind_attribute
      -- index_list for iir_kind_indexed_name
      -- index_list for iir_kind_array_type_definition
      -- index_list for iir_kind_array_subtype_definition
      -- target for iir_kind_signal_assignment_statement
      -- target for iir_kind_variable_assignment_statement
      -- time for iir_kind_waveform_element
      -- target for iir_kind_concurrent_conditional_signal_assignment
      -- target for iir_kind_concurrent_selected_signal_assignment
      -- assertion_condition for iir_kind_concurrent_assertion_statement
      -- assertion_condition for iir_kind_assertion_statement
      -- null_iir for iir_kind_conditional_waveform
      -- sequential_statement_chain for iir_kind_if_statement
      -- sequential_statement_chain for iir_kind_elsif
      -- sequential_statement_chain for iir_kind_sensitized_process_statement
      -- sequential_statement_chain for iir_kind_process_statement
      -- sequential_statement_chain for iir_kind_for_loop_statement
      -- sequential_statement_chain for iir_kind_while_loop_statement
      -- sequential_statement_chain for iir_kind_function_Body
      -- sequential_statement_chain for iir_kind_function_Spec_Body
      -- sequential_statement_chain for iir_kind_procedure_Body
      -- sequential_statement_chain for iir_kind_procedure_Spec_Body
      -- name for iir_kind_object_alias_declaration
      -- name for iir_kind_physical_int_literal
      -- name for iir_kind_physical_fp_literal
      -- name for iir_kind_association_choice_by_name
      -- name for iir_kind_group_declaration
      -- default_value for iir_kind_signal_declaration
      -- default_value for iir_kind_guard_signal_declaration
      -- default_value for iir_kind_variable_declaration
      -- default_value for iir_kind_constant_declaration
      -- default_value for iir_kind_signal_interface_declaration
      -- default_value for iir_kind_variable_interface_declaration
      -- default_value for iir_kind_constant_interface_declaration
      -- default_value for iir_kind_file_interface_declaration
      -- guard_expression for iir_kind_guard_signal_declaration
      -- operand for iir_kinds_monadic_operator
      -- left for iir_kinds_dyadic_operator
      -- actual for iir_kind_association_element_by_expression
      -- instantiated_unit for Iir_Kind_Component_Instantiation_Statement
      -- parameter_association_chain for iir_kind_function_call
      -- parameter_association_chain for iir_kind_procedure_call
      -- parameter_association_chain for iir_kind_concurrent_procedure_call_st.
      -- library_unit for iir_kind_design_unit
      -- multiplier for iir_kind_unit_declaration
      -- primary_unit for iir_kind_physical_type_definition
      -- condition_clause for iir_kind_wait_statement
      -- element_declaration_list for iir_kind_record_type_definition
      -- loop for iir_kind_exit_statement
      -- loop for iir_kind_next_statement
      -- file_logical_name for iir_kind_file_declaration
      -- configuration_item_chain for iir_kind_block_configuration
      -- architecture for iir_kind_entity_aspect_entity
      -- library_declaration for iir_kind_library_clause
      -- attribute_designator for iir_kind_attribute_specification
      -- attribute_specification for iir_kind_attribute_value
      -- signal_list for iir_kind_disconnection_specification
      -- generation_scheme for iir_kind_generate_statement
      -- incomplete_type_List for iir_kind_incomplete_type_definition
      -- file_time_stamp for iir_kind_design_file
      -- default_generic_map_aspect_list for iir_kind_binding_indication
      -- aggr_low_limit for iir_kind_aggregate_info
      -- enumeration_decl for iir_kind_enumeration_literal
      -- simple_aggregate_list for iir_kind_simple_aggregate
      Field2: Iir := Null_Iir;

      -- Usages of field3:
      -- dependence_list for iir_kind_design_unit
      -- block_statement for iir_kind_signal_declaration
      -- block_statement for iir_kind_guard_signal_declaration
      -- subprogram_declaration for iir_kind_function_Spec_Body
      -- subprogram_declaration for iir_kind_function_Body
      -- subprogram_declaration for iir_kind_Procedure_Spec_Body
      -- subprogram_declaration for iir_kind_Procedure_Body
      -- body for iir_kind_function_specification
      -- body for iir_kind_procedure_specification
      -- declaration_list for iir_kind_entity_declaration
      -- declaration_list for iir_kind_architecture_declaration
      -- declaration_list for iir_kind_configuration_declaration
      -- declaration_list for iir_kind_block_statement
      -- declaration_list for iir_kind_package_declaration
      -- declaration_list for iir_kind_package_body
      -- declaration_list for iir_kind_sensitized_process_statement
      -- declaration_list for iir_kind_process_statement
      -- declaration_list for iir_kind_block_configuration
      -- declaration_list for iir_kind_generate_statement
      -- enumeration_literal_list for iir_enumeration_type_definition
      -- right_limit for iir_kind_range_expression
      -- element_subtype for iir_array_type_definition
      -- element_subtype for iir_array_subtype_definition
      -- report_expression for iir_kind_concurrent_assertion_statement
      -- report_expression for iir_kind_assertion_statement
      -- report_expression for iir_kind_report_statement
      -- waveform_chain for iir_kind_signal_assignment_statement
      -- conditional_waveform_chain for iir_kind_conc_conditional_signal_assign
      -- waveform_chain for iir_kind_conditional_waveform
      -- else_clause for iir_kind_if_statement
      -- else_clause for iir_kind_elsif
      -- expression of iir_kind_concurrent_selected_signal_assignment
      -- expression of iir_kind_variable_assignment_statement
      -- prefix for iir_kinds_attribute
      -- prefix for iir_kind_indexed_name
      -- prefix for iir_kind_slice_name
      -- prefix for iir_kind_selected_name
      -- prefix for iir_kind_selected_by_all_name
      -- prefix for iir_kind_parenthesis_name
      -- prefix for iir_kind_selected_element
      -- prefix for iir_kind_implicit_dereference
      -- port_map_aspect for Iir_Kind_Component_Instantiation_Statement
      -- port_map_aspect for Iir_Kind_binding_indication
      -- port_map_aspect for Iir_Kind_block_header
      -- binding_indication for iir_kind_Component_configuration
      -- binding_indication for Iir_Kind_Configuration_specifiation
      -- expression for iir_kind_return_statement
      -- expression for iir_kind_association_choice_by_expression
      -- expression for iir_kind_case_statement
      -- expression for iir_kind_qualified_expression
      -- expression for iir_kind_type_conversion
      -- expression for iir_kind_allocator_by_expression
      -- expression for iir_kind_allocator_by_subtype
      -- expression for iir_kind_attribute_specification
      -- expression for iir_kind_disconnection_specification
      -- unit_chain for iir_kind_physical_type_definition
      -- timeout_clause for iir_kind_wait_statement
      -- file_open_kind for iir_kind_file_declaration
      -- designated_entity for iir_kind_attribute_value
      -- associated_formal for iir_kinds_association_element
      -- deferred_declaration for iir_kind_constant_declaration
      -- literal_origin for iir_kind_character_literal
      -- literal_origin for iir_kind_string_literal
      -- literal_origin for iir_kind_bit_string_literal
      -- literal_origin for iir_kind_integer_literal
      -- literal_origin for iir_kind_floating_point_literal
      -- literal_origin for iir_kind_physical_int_literal
      -- literal_origin for iir_kind_physical_fp_literal
      -- literal_origin for iir_kind_enumeration_literal
      -- analysis_time_stamp for iir_kind_design_file
      -- aggr_high_limit for iir_kind_aggregate_info
      -- aggregate_info for iir_kind_aggregate
      -- implementation for iir_kind_function_call
      -- implementation for iir_kind_procedure_call
      -- implementation for iir_kind_concurrent_procedure_call_statement
      -- implementation for iir_kind_dyadic_operator
      -- implementation for iir_kind_monadic_operator
      Field3: Iir := Null_Iir;

      -- Usages of field4:
      -- design_file for iir_kind_design_unit
      -- generic_chain for iir_kind_entity_declaration
      -- generic_chain for iir_kind_component_declaration
      -- generic_chain for iir_kind_block_header
      -- base_type for iir_kind_integer_type_definition
      -- base_type for iir_kind_integer_subtype_definition
      -- base_type for iir_kind_floating_type_definition
      -- base_type for iir_kind_floating_subtype_definition
      -- base_type for iir_kind_subtype_definition
      -- base_type for iir_kind_enumeration_type_definition
      -- base_type for iir_kind_enumeration_subtype_definition
      -- base_type for iir_kind_array_type_definition
      -- base_type for iir_kind_array_subtype_definition
      -- base_type for iir_kind_unconstrained_array_subtype_definition
      -- base_type for iir_kind_range_attribute
      -- base_type for iir_kind_physical_type_definition
      -- base_type for iir_kind_physical_subtype_definition
      -- base_type for iir_kind_record_type_definition
      -- base_type for iir_kind_record_subtype_definition
      -- base_type for iir_kind_access_type_definition
      -- base_type for iir_kind_access_subtype_definition
      -- base_type for iir_kind_incomplete_type_definition
      -- base_type for iir_kind_file_type_definition
      -- severity_expression for iir_kind_concurrent_assertion_statement
      -- severity_expression for iir_kind_assertion_statement
      -- severity_expression for iir_kind_report_statement
      -- sensitivity_list for iir_kind_sensitized_process_statement
      -- sensitivity_list for iir_kind_wait_statement
      -- name_value of iir_kind_simple_name
      -- association_chain for iir_kind_association_element_by_individual
      -- association_chain for iir_kind_parenthesis_name
      -- association_choices_list for iir_kind_aggregate
      -- association_choices_list for iir_kind_case_statement
      -- guard for iir_kind_concurrent_conditional_signal_assignment
      -- guard for iir_kind_concurrent_selected_signal_assignment
      -- entity_aspect for iir_kind_binding_indication
      -- default_binding_indicat for iir_kind_component_instantiation_statement
      -- component_name for iir_kind_component_configuration
      -- component_name for iir_kind_configuration_specification
      -- prev_block_configuration for iir_kind_block_configuration
      -- interface_declaration for iir_kind_function_Specification
      -- interface_declaration for iir_kind_function_Spec_Body
      -- interface_declaration for iir_kind_procedure_Specification
      -- interface_declaration for iir_kind_procedure_Spec_Body
      -- interface_declaration for iir_kind_implicit_function_declaration
      -- interface_declaration for iir_kind_implicit_procedure_declaration
      -- subprogram_specification for iir_kind_function_Body
      -- subprogram_specification for iir_kind_procedure_Body
      -- in_conversion for iir_kind_association_element_by_expression
      -- default_configuration for iir_kind_architecture_declaration
      -- bit_string_0 for iir_kind_bit_string_literal
      -- base_name for iir_kind_object_alias_declaration
      -- base_name for iir_kind_signal_declaration
      -- base_name for iir_kind_guard_signal_declaration
      -- base_name for iir_kind_variable_declaration
      -- base_name for iir_kind_file_declaration
      -- base_name for iir_kind_constant_declaration
      -- base_name for iir_kind_iterator_declaration
      -- base_name for iir_kind_slice_name
      -- base_name for iir_kind_indexed_name
      -- base_name for iir_kind_selected_element
      -- base_name for iir_kind_selected_by_all_name
      -- base_name for iir_kind_implicit_dereference
      -- base_name for iir_kind_attribute_value
      -- base_name for iir_kind_function_call
      -- block_block_configuration for iir_kind_block_statement
      -- right for iir_kinds_dyadic_operator
      Field4: Iir := Null_Iir;

      -- Usages of field5 (aka nbr1).
      -- driver_list for iir_kind_sensitized_process_statement
      -- driver_list for iir_kind_process_statement
      -- driver_list for iir_kinds_function_specification
      -- driver_list for iir_kinds_procedure_specification
      -- guard_sensitivity_list for iir_kind_guard_signal_declaration
      -- signal_driver for iir_kind_signal_declaration
      -- reject_time for iir_kind_concurrent_selected_signal_assignment
      -- reject_time for iir_kind_concurrent_conditionnal_signal_assignment
      -- reject_time for iir_kind_signal_assignment_statement
      -- resolution_function for iir_kind_integer_subtype_definition
      -- resolution_function for iir_kind_floating_subtype_definition
      -- resolution_function for iir_kind_enumeration_subtype_definition
      -- resolution_function for iir_kind_physical_subtype_definition
      -- resolution_function for iir_kind_array_subtype_definition
      -- resolution_function for iir_kind_unconstrained_array_subtype_definit.
      -- resolution_function for iir_kind_record_subtype_definition
      -- date for iir_kind_library_declaration
      -- date for iir_kind_design_unit
      -- generic_map_aspect for Iir_Kind_Component_Instantiation_Statement
      -- generic_map_aspect for Iir_Kind_block_header
      -- generic_map_aspect for Iir_Kind_binding_indication
      -- generation_scheme for iir_kind_generate_statement
      -- design_unit for iir_kind_constant_declaration
      -- design_unit for iir_kind_entity_declaration
      -- design_unit for iir_kind_configuration_declaration
      -- design_unit for iir_kind_package_declaration
      -- design_unit for iir_kind_body_declaration
      -- design_unit for iir_kind_architecture_declaration
      -- out_conversion for iir_kind_association_element_by_expression
      -- bit_string_1 for iir_kind_bit_string_literal
      Field5: Iir := Null_Iir;

      -- Usages of Field6:
      -- offset for iir_kind_design_unit
      -- number of element for iir_kinds_list
      -- base for iir_kind_bit_string_literal
      -- element_position for iir_kind_element_declaration
      -- type_mark for iir_kind_qualified_expression
      -- type_mark for iir_kind_file_type_definition
      -- type_mark for iir_kind_integer_subtype_definition
      -- type_mark for iir_kind_floating_subtype_definition
      -- type_mark for iir_kind_enumeration_subtype_definition
      -- type_mark for iir_kind_physical_subtype_definition
      -- type_mark for iir_kind_access_subtype_definition
      -- type_mark for iir_kind_record_subtype_definition
      -- type_mark for iir_kind_unconstrained_array_subtype_definition
      -- bit_string_base for iir_kind_bit_string_literal
      -- default_port_map_aspect_list for iir_kind_binding_indication

      -- Usages of nbr3/field7:
      -- line for iir_kind_design_unit
      -- max number of elements for iir_kinds_list
      -- implicit_definition for iir_kind_implicit_function_declaration
      -- implicit_definition for iir_kind_implicit_procedure_declaration
      -- block_header for iir_kind_block_statement
      -- delay_mechanism for iir_kind_concurrent_selected_signal_assignment
      -- delay_mechanism for iir_kind_concurrent_conditionnal_signal_assignment
      -- delay_mechanism for iir_kind_signal_assignment_statement
      -- value for iir_kind_integer_literal
      -- value for iir_kind_enumeration_literal
      -- value for iir_kind_unit_declaration
      -- value for iir_kind_physical_int_literal
      -- fp_value for iir_kind_physical_fp_literal
      -- fp_value for iir_kind_floating_point_literal
      -- entity_kind for iir_kind_entity_class
      -- entity_kind for iir_kind_attribute_specification
      -- callees_list for iir_kind_process_declaration
      -- callees_list for iir_kind_sensitized_process_declaration
      -- library_directory for iir_kind_library_declaration
      -- filename for iir_kind_design_file
      -- directory for iir_kind_design_file
      -- aggr_max_length for iir_kind_aggregate_info
      case Kind is
         when Iir_Kind_Floating_Point_Literal
           | Iir_Kind_Physical_Fp_Literal =>
            Nbr3_Fp64 : Iir_Fp64;
         when Iir_Kind_Integer_Literal
           | Iir_Kind_Enumeration_Literal
           | Iir_Kind_Physical_Int_Literal
           | Iir_Kind_Unit_Declaration =>
            Nbr3_Int64 : Iir_Int64;
         when others =>
            Nbr2: Integer := 0;
            Nbr3: Integer := 0;
      end case;
   end record;

   pragma Pack (Iir_Node);
end Iirs;
